@@ -97,7 +97,8 @@ mc_config_get_string (mc_config_t * mc_config, const gchar * group,
 
     if (!mc_config_has_param (mc_config, group, param))
     {
-        mc_config_set_string (mc_config, group, param, def ? def : "");
+        if (def != NULL)
+            mc_config_set_string (mc_config, group, param, def);
         return g_strdup (def);
     }
 
@@ -140,7 +141,8 @@ mc_config_get_string_raw (const mc_config_t * mc_config, const gchar * group,
 
     if (!mc_config_has_param (mc_config, group, param))
     {
-        mc_config_set_string (mc_config, group, param, def ? def : "");
+        if (def != NULL)
+            mc_config_set_string (mc_config, group, param, def);
         return g_strdup (def);
     }
 
