@@ -34,6 +34,10 @@
 #include <stdlib.h>
 #include <stdarg.h>
 #include <signal.h>
+#ifdef HAVE_SYS_IOCTL_H
+#include <sys/ioctl.h>
+#endif
+#include <termios.h>
 
 #include "lib/global.h"
 #include "lib/strutil.h"        /* str_term_form */
@@ -105,7 +109,7 @@ sigwinch_handler (int dummy)
 {
     (void) dummy;
 
-    mc_global.tty.winch_flag = TRUE;
+    mc_global.tty.winch_flag = 1;
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -199,7 +203,6 @@ tty_init (gboolean mouse_enable, gboolean is_xterm)
     if (!mouse_enable)
         use_mouse_p = MOUSE_DISABLED;
     tty_init_xterm_support (is_xterm);  /* do it before do_enter_ca_mode() call */
-    init_mouse ();
     do_enter_ca_mode ();
     tty_raw_mode ();
     noecho ();
@@ -224,6 +227,41 @@ tty_shutdown (void)
 
 /* --------------------------------------------------------------------------------------------- */
 
+void
+tty_change_screen_size (void)
+{
+#if defined(TIOCGWINSZ) && NCURSES_VERSION_MAJOR >= 4
+    struct winsize winsz;
+
+    winsz.ws_col = winsz.ws_row = 0;
+
+#ifndef NCURSES_VERSION
+    tty_noraw_mode ();
+    tty_reset_screen ();
+#endif
+
+    /* Ioctl on the STDIN_FILENO */
+    ioctl (fileno (stdout), TIOCGWINSZ, &winsz);
+    if (winsz.ws_col != 0 && winsz.ws_row != 0)
+    {
+#if defined(NCURSES_VERSION) && defined(HAVE_RESIZETERM)
+        resizeterm (winsz.ws_row, winsz.ws_col);
+        clearok (stdscr, TRUE); /* sigwinch's should use a semaphore! */
+#else
+        COLS = winsz.ws_col;
+        LINES = winsz.ws_row;
+#endif
+    }
+#endif /* defined(TIOCGWINSZ) || NCURSES_VERSION_MAJOR >= 4 */
+
+#ifdef ENABLE_SUBSHELL
+    if (mc_global.tty.use_subshell)
+        tty_resize (mc_global.tty.subshell_pty);
+#endif
+}
+
+/* --------------------------------------------------------------------------------------------- */
+
 void
 tty_reset_prog_mode (void)
 {
