@@ -62,7 +62,7 @@ int mc_tty_frm[MC_TTY_FRM_MAX];
 
 /*** file scope variables ************************************************************************/
 
-static volatile sig_atomic_t got_interrupt = 0;
+static SIG_ATOMIC_VOLATILE_T got_interrupt = 0;
 
 /*** file scope functions ************************************************************************/
 /* --------------------------------------------------------------------------------------------- */
@@ -90,6 +90,7 @@ gboolean
 tty_check_term (gboolean force_xterm)
 {
     const char *termvalue;
+    const char *xdisplay;
 
     termvalue = getenv ("TERM");
     if (termvalue == NULL || *termvalue == '\0')
@@ -98,10 +99,16 @@ tty_check_term (gboolean force_xterm)
         exit (EXIT_FAILURE);
     }
 
+    xdisplay = getenv ("DISPLAY");
+    if (xdisplay != NULL && *xdisplay == '\0')
+        xdisplay = NULL;
+
     return force_xterm || strncmp (termvalue, "xterm", 5) == 0
         || strncmp (termvalue, "konsole", 7) == 0
         || strncmp (termvalue, "rxvt", 4) == 0
-        || strcmp (termvalue, "Eterm") == 0 || strcmp (termvalue, "dtterm") == 0;
+        || strcmp (termvalue, "Eterm") == 0
+        || strcmp (termvalue, "dtterm") == 0
+        || (strncmp (termvalue, "screen", 6) == 0 && xdisplay != NULL);
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -252,38 +259,6 @@ tty_resize (int fd)
 
 /* --------------------------------------------------------------------------------------------- */
 
-void
-tty_change_screen_size (void)
-{
-#if defined(HAVE_SLANG) || NCURSES_VERSION_MAJOR >= 4
-#if defined TIOCGWINSZ
-    struct winsize winsz;
-
-    winsz.ws_col = winsz.ws_row = 0;
-    /* Ioctl on the STDIN_FILENO */
-    ioctl (0, TIOCGWINSZ, &winsz);
-    if (winsz.ws_col && winsz.ws_row)
-    {
-#if defined(NCURSES_VERSION) && defined(HAVE_RESIZETERM)
-        resizeterm (winsz.ws_row, winsz.ws_col);
-        clearok (stdscr, TRUE); /* sigwinch's should use a semaphore! */
-#else
-        COLS = winsz.ws_col;
-        LINES = winsz.ws_row;
-#endif
-#ifdef HAVE_SUBSHELL_SUPPORT
-        if (!mc_global.tty.use_subshell)
-            return;
-
-        tty_resize (mc_global.tty.subshell_pty);
-#endif
-    }
-#endif /* TIOCGWINSZ */
-#endif /* defined(HAVE_SLANG) || NCURSES_VERSION_MAJOR >= 4 */
-}
-
-/* --------------------------------------------------------------------------------------------- */
-
 void
 tty_init_xterm_support (gboolean is_xterm)
 {
@@ -335,6 +310,10 @@ tty_init_xterm_support (gboolean is_xterm)
             }
         }
     }
+
+    /* No termcap for SGR extended mouse (yet), hardcode it for now */
+    if (xmouse_seq != NULL)
+        xmouse_extended_seq = ESC_STR "[<";
 }
 
 /* --------------------------------------------------------------------------------------------- */
