@@ -272,10 +272,6 @@ void vfs_free_handle (int handle);
 void vfs_setup_cwd (void);
 char *_vfs_get_cwd (void);
 
-#ifdef HAVE_CHARSET
-vfs_path_t *vfs_change_encoding (vfs_path_t * vpath, const char *encoding);
-#endif
-
 int vfs_preallocate (int dest_desc, off_t src_fsize, off_t dest_fsize);
 
 /**
