@@ -2,7 +2,7 @@
    Dialog box features module for the Midnight Commander
 
    Copyright (C) 1994, 1995, 1998, 1999, 2000, 2001, 2002, 2003, 2004,
-   2005, 2007, 2009, 2010, 2011
+   2005, 2007, 2009, 2010, 2011, 2013
    The Free Software Foundation, Inc.
 
    This file is part of the Midnight Commander.
@@ -85,7 +85,7 @@ typedef enum
 /*** file scope functions ************************************************************************/
 
 static GList *
-dlg_widget_next (Dlg_head * h, GList * l)
+dlg_widget_next (WDialog * h, GList * l)
 {
     GList *next;
 
@@ -99,7 +99,7 @@ dlg_widget_next (Dlg_head * h, GList * l)
 /* --------------------------------------------------------------------------------------------- */
 
 static GList *
-dlg_widget_prev (Dlg_head * h, GList * l)
+dlg_widget_prev (WDialog * h, GList * l)
 {
     GList *prev;
 
@@ -118,7 +118,7 @@ dlg_widget_prev (Dlg_head * h, GList * l)
  */
 
 static void
-dlg_broadcast_msg_to (Dlg_head * h, widget_msg_t msg, gboolean reverse, int flags)
+dlg_broadcast_msg_to (WDialog * h, widget_msg_t msg, gboolean reverse, int flags)
 {
     GList *p, *first;
 
@@ -137,7 +137,7 @@ dlg_broadcast_msg_to (Dlg_head * h, widget_msg_t msg, gboolean reverse, int flag
 
     do
     {
-        Widget *w = (Widget *) p->data;
+        Widget *w = WIDGET (p->data);
 
         if (reverse)
             p = dlg_widget_prev (h, p);
@@ -145,7 +145,7 @@ dlg_broadcast_msg_to (Dlg_head * h, widget_msg_t msg, gboolean reverse, int flag
             p = dlg_widget_next (h, p);
 
         if ((flags == 0) || ((flags & w->options) != 0))
-            send_message (w, msg, 0);
+            send_message (w, NULL, msg, 0, NULL);
     }
     while (first != p);
 }
@@ -156,7 +156,7 @@ dlg_broadcast_msg_to (Dlg_head * h, widget_msg_t msg, gboolean reverse, int flag
   * Read histories from the ${XDG_CACHE_HOME}/mc/history file
   */
 static void
-dlg_read_history (Dlg_head * h)
+dlg_read_history (WDialog * h)
 {
     char *profile;
     ev_history_load_save_t event_data;
@@ -165,7 +165,7 @@ dlg_read_history (Dlg_head * h)
         return;
 
     profile = mc_config_get_full_path (MC_HISTORY_FILE);
-    event_data.cfg = mc_config_init (profile);
+    event_data.cfg = mc_config_init (profile, TRUE);
     event_data.receiver = NULL;
 
     /* create all histories in dialog */
@@ -178,16 +178,16 @@ dlg_read_history (Dlg_head * h)
 /* --------------------------------------------------------------------------------------------- */
 
 static gboolean
-dlg_unfocus (Dlg_head * h)
+dlg_unfocus (WDialog * h)
 {
     /* we can unfocus disabled widget */
     if ((h->current != NULL) && (h->state == DLG_CONSTRUCT || h->state == DLG_ACTIVE))
     {
-        Widget *current = (Widget *) h->current->data;
+        Widget *current = WIDGET (h->current->data);
 
-        if (send_message (current, WIDGET_UNFOCUS, 0) == MSG_HANDLED)
+        if (send_message (current, NULL, MSG_UNFOCUS, 0, NULL) == MSG_HANDLED)
         {
-            h->callback (h, current, DLG_UNFOCUS, 0, NULL);
+            send_message (h, current, MSG_UNFOCUS, 0, NULL);
             return TRUE;
         }
     }
@@ -200,8 +200,8 @@ dlg_unfocus (Dlg_head * h)
 static int
 dlg_find_widget_callback (const void *a, const void *b)
 {
-    const Widget *w = (const Widget *) a;
-    callback_fn f = (callback_fn) b;
+    const Widget *w = WIDGET (a);
+    widget_cb_fn f = (widget_cb_fn) b;
 
     return (w->callback == f) ? 0 : 1;
 }
@@ -213,9 +213,9 @@ dlg_find_widget_callback (const void *a, const void *b)
  */
 
 static void
-do_select_widget (Dlg_head * h, GList * w, select_dir_t dir)
+do_select_widget (WDialog * h, GList * w, select_dir_t dir)
 {
-    Widget *w0 = (Widget *) h->current->data;
+    Widget *w0 = WIDGET (h->current->data);
 
     if (!dlg_unfocus (h))
         return;
@@ -244,12 +244,12 @@ do_select_widget (Dlg_head * h, GList * w, select_dir_t dir)
             break;
         }
     }
-    while (h->current != w /* && (((Widget *) h->current->data)->options & W_DISABLED) == 0 */ );
+    while (h->current != w /* && (WIDGET (h->current->data)->options & W_DISABLED) == 0 */ );
 
-    if (dlg_overlap (w0, (Widget *) h->current->data))
+    if (widget_overlapped (w0, WIDGET (h->current->data)))
     {
-        send_message ((Widget *) h->current->data, WIDGET_DRAW, 0);
-        send_message ((Widget *) h->current->data, WIDGET_FOCUS, 0);
+        send_message (h->current->data, NULL, MSG_DRAW, 0, NULL);
+        send_message (h->current->data, NULL, MSG_FOCUS, 0, NULL);
     }
 }
 
@@ -271,7 +271,7 @@ refresh_cmd (void)
 /* --------------------------------------------------------------------------------------------- */
 
 static cb_ret_t
-dlg_execute_cmd (Dlg_head * h, unsigned long command)
+dlg_execute_cmd (WDialog * h, unsigned long command)
 {
     cb_ret_t ret = MSG_HANDLED;
     switch (command)
@@ -338,7 +338,7 @@ dlg_execute_cmd (Dlg_head * h, unsigned long command)
 /* --------------------------------------------------------------------------------------------- */
 
 static cb_ret_t
-dlg_handle_key (Dlg_head * h, int d_key)
+dlg_handle_key (WDialog * h, int d_key)
 {
     unsigned long command;
 
@@ -347,7 +347,7 @@ dlg_handle_key (Dlg_head * h, int d_key)
     if (command == CK_IgnoreKey)
         return MSG_NOT_HANDLED;
 
-    if (h->callback (h, NULL, DLG_ACTION, command, NULL) == MSG_HANDLED
+    if (send_message (h, NULL, MSG_ACTION, command, NULL) == MSG_HANDLED
         || dlg_execute_cmd (h, command) == MSG_HANDLED)
         return MSG_HANDLED;
 
@@ -357,52 +357,50 @@ dlg_handle_key (Dlg_head * h, int d_key)
 /* --------------------------------------------------------------------------------------------- */
 
 static int
-dlg_mouse_event (Dlg_head * h, Gpm_Event * event)
+dlg_mouse_event (WDialog * h, Gpm_Event * event)
 {
-    GList *item;
-    GList *starting_widget = h->current;
-    int x = event->x;
-    int y = event->y;
+    Widget *wh = WIDGET (h);
 
-    /* close the dialog by mouse click out of dialog area */
-    if (mouse_close_dialog && !h->fullscreen && ((event->buttons & GPM_B_LEFT) != 0) && ((event->type & GPM_DOWN) != 0) /* left click */
-        && !((x > h->x) && (x <= h->x + h->cols) && (y > h->y) && (y <= h->y + h->lines)))
+    GList *p, *first;
+
+    /* close the dialog by mouse left click out of dialog area */
+    if (mouse_close_dialog && !h->fullscreen && ((event->buttons & GPM_B_LEFT) != 0)
+        && ((event->type & GPM_DOWN) != 0) && !mouse_global_in_widget (event, wh))
     {
         h->ret_value = B_CANCEL;
         dlg_stop (h);
         return MOU_NORMAL;
     }
 
-    if (h->mouse != NULL)
+    if (wh->mouse != NULL)
     {
         int mou;
 
-        mou = h->mouse (event, h);
+        mou = wh->mouse (event, wh);
         if (mou != MOU_UNHANDLED)
             return mou;
     }
 
-    item = starting_widget;
+    first = h->current;
+    p = first;
+
     do
     {
-        Widget *widget = (Widget *) item->data;
+        Widget *w = WIDGET (p->data);
 
-        if ((h->flags & DLG_REVERSE) == 0)
-            item = dlg_widget_prev (h, item);
-        else
-            item = dlg_widget_next (h, item);
+        p = dlg_widget_prev (h, p);
 
-        if ((widget->options & W_DISABLED) == 0 && widget->mouse != NULL)
+        if ((w->options & W_DISABLED) == 0 && w->mouse != NULL)
         {
             /* put global cursor position to the widget */
             int ret;
 
-            ret = widget->mouse (event, widget);
+            ret = w->mouse (event, w);
             if (ret != MOU_UNHANDLED)
                 return ret;
         }
     }
-    while (item != starting_widget);
+    while (p != first);
 
     return MOU_UNHANDLED;
 }
@@ -410,7 +408,7 @@ dlg_mouse_event (Dlg_head * h, Gpm_Event * event)
 /* --------------------------------------------------------------------------------------------- */
 
 static cb_ret_t
-dlg_try_hotkey (Dlg_head * h, int d_key)
+dlg_try_hotkey (WDialog * h, int d_key)
 {
     GList *hot_cur;
     Widget *current;
@@ -428,7 +426,7 @@ dlg_try_hotkey (Dlg_head * h, int d_key)
      * the currently selected widget is an input line
      */
 
-    current = (Widget *) h->current->data;
+    current = WIDGET (h->current->data);
 
     if ((current->options & W_DISABLED) != 0)
         return MSG_NOT_HANDLED;
@@ -448,7 +446,7 @@ dlg_try_hotkey (Dlg_head * h, int d_key)
 
     handled = MSG_NOT_HANDLED;
     if ((current->options & W_WANT_HOTKEY) != 0)
-        handled = send_message (current, WIDGET_HOTKEY, d_key);
+        handled = send_message (current, NULL, MSG_HOTKEY, d_key, NULL);
 
     /* If not used, send hotkey to other widgets */
     if (handled == MSG_HANDLED)
@@ -459,10 +457,10 @@ dlg_try_hotkey (Dlg_head * h, int d_key)
     /* send it to all widgets */
     while (h->current != hot_cur && handled == MSG_NOT_HANDLED)
     {
-        current = (Widget *) hot_cur->data;
+        current = WIDGET (hot_cur->data);
 
         if ((current->options & W_WANT_HOTKEY) != 0 && (current->options & W_DISABLED) == 0)
-            handled = send_message (current, WIDGET_HOTKEY, d_key);
+            handled = send_message (current, NULL, MSG_HOTKEY, d_key, NULL);
 
         if (handled == MSG_NOT_HANDLED)
             hot_cur = dlg_widget_next (h, hot_cur);
@@ -477,7 +475,7 @@ dlg_try_hotkey (Dlg_head * h, int d_key)
 /* --------------------------------------------------------------------------------------------- */
 
 static void
-dlg_key_event (Dlg_head * h, int d_key)
+dlg_key_event (WDialog * h, int d_key)
 {
     cb_ret_t handled;
 
@@ -503,32 +501,32 @@ dlg_key_event (Dlg_head * h, int d_key)
     }
 
     /* first can dlg_callback handle the key */
-    handled = h->callback (h, NULL, DLG_KEY, d_key, NULL);
+    handled = send_message (h, NULL, MSG_KEY, d_key, NULL);
 
     /* next try the hotkey */
     if (handled == MSG_NOT_HANDLED)
         handled = dlg_try_hotkey (h, d_key);
 
     if (handled == MSG_HANDLED)
-        h->callback (h, NULL, DLG_HOTKEY_HANDLED, 0, NULL);
+        send_message (h, NULL, MSG_HOTKEY_HANDLED, 0, NULL);
     else
         /* not used - then try widget_callback */
-        handled = send_message ((Widget *) h->current->data, WIDGET_KEY, d_key);
+        handled = send_message (h->current->data, NULL, MSG_KEY, d_key, NULL);
 
     /* not used- try to use the unhandled case */
     if (handled == MSG_NOT_HANDLED)
-        handled = h->callback (h, NULL, DLG_UNHANDLED_KEY, d_key, NULL);
+        handled = send_message (h, NULL, MSG_UNHANDLED_KEY, d_key, NULL);
 
     if (handled == MSG_NOT_HANDLED)
         handled = dlg_handle_key (h, d_key);
 
-    h->callback (h, NULL, DLG_POST_KEY, d_key, NULL);
+    send_message (h, NULL, MSG_POST_KEY, d_key, NULL);
 }
 
 /* --------------------------------------------------------------------------------------------- */
 
 static void
-frontend_run_dlg (Dlg_head * h)
+frontend_dlg_run (WDialog * h)
 {
     int d_key;
     Gpm_Event event;
@@ -538,13 +536,13 @@ frontend_run_dlg (Dlg_head * h)
     /* close opened editors, viewers, etc */
     if (!h->modal && mc_global.midnight_shutdown)
     {
-        h->callback (h, NULL, DLG_VALIDATE, 0, NULL);
+        send_message (h, NULL, MSG_VALIDATE, 0, NULL);
         return;
     }
 
     while (h->state == DLG_ACTIVE)
     {
-        if (mc_global.tty.winch_flag)
+        if (mc_global.tty.winch_flag != 0)
             dialog_change_screen_size ();
 
         if (is_idle ())
@@ -552,8 +550,8 @@ frontend_run_dlg (Dlg_head * h)
             if (idle_hook)
                 execute_hooks (idle_hook);
 
-            while ((h->flags & DLG_WANT_IDLE) && is_idle ())
-                h->callback (h, NULL, DLG_IDLE, 0, NULL);
+            while ((WIDGET (h)->options & W_WANT_IDLE) != 0 && is_idle ())
+                send_message (h, NULL, MSG_IDLE, 0, NULL);
 
             /* Allow terminating the dialog from the idle handler */
             if (h->state != DLG_ACTIVE)
@@ -569,7 +567,7 @@ frontend_run_dlg (Dlg_head * h)
         dlg_process_event (h, d_key, &event);
 
         if (h->state == DLG_CLOSED)
-            h->callback (h, NULL, DLG_VALIDATE, 0, NULL);
+            send_message (h, NULL, MSG_VALIDATE, 0, NULL);
     }
 }
 
@@ -578,7 +576,7 @@ frontend_run_dlg (Dlg_head * h)
 static int
 dlg_find_widget_by_id (gconstpointer a, gconstpointer b)
 {
-    Widget *w = (Widget *) a;
+    Widget *w = WIDGET (a);
     unsigned long id = GPOINTER_TO_UINT (b);
 
     return w->id == id ? 0 : 1;
@@ -588,19 +586,12 @@ dlg_find_widget_by_id (gconstpointer a, gconstpointer b)
 /*** public functions ****************************************************************************/
 /* --------------------------------------------------------------------------------------------- */
 
-/** draw box in window */
-void
-draw_box (Dlg_head * h, int y, int x, int ys, int xs, gboolean single)
-{
-    tty_draw_box (h->y + y, h->x + x, ys, xs, single);
-}
-
-/* --------------------------------------------------------------------------------------------- */
-
 /** Clean the dialog area, draw the frame and the title */
 void
-common_dialog_repaint (Dlg_head * h)
+dlg_default_repaint (WDialog * h)
 {
+    Widget *wh = WIDGET (h);
+
     int space;
 
     if (h->state != DLG_ACTIVE)
@@ -610,12 +601,12 @@ common_dialog_repaint (Dlg_head * h)
 
     tty_setcolor (h->color[DLG_COLOR_NORMAL]);
     dlg_erase (h);
-    draw_box (h, space, space, h->lines - 2 * space, h->cols - 2 * space, FALSE);
+    tty_draw_box (wh->y + space, wh->x + space, wh->lines - 2 * space, wh->cols - 2 * space, FALSE);
 
     if (h->title != NULL)
     {
         tty_setcolor (h->color[DLG_COLOR_TITLE]);
-        dlg_move (h, space, (h->cols - str_term_width1 (h->title)) / 2);
+        widget_move (h, space, (wh->cols - str_term_width1 (h->title)) / 2);
         tty_print_string (h->title);
     }
 }
@@ -624,22 +615,24 @@ common_dialog_repaint (Dlg_head * h)
 /** this function allows to set dialog position */
 
 void
-dlg_set_position (Dlg_head * h, int y1, int x1, int y2, int x2)
+dlg_set_position (WDialog * h, int y1, int x1, int y2, int x2)
 {
+    Widget *wh = WIDGET (h);
+
     /* save old positions, will be used to reposition childs */
     int ox, oy, oc, ol;
     int shift_x, shift_y, scale_x, scale_y;
 
     /* save old positions, will be used to reposition childs */
-    ox = h->x;
-    oy = h->y;
-    oc = h->cols;
-    ol = h->lines;
+    ox = wh->x;
+    oy = wh->y;
+    oc = wh->cols;
+    ol = wh->lines;
 
-    h->x = x1;
-    h->y = y1;
-    h->lines = y2 - y1;
-    h->cols = x2 - x1;
+    wh->x = x1;
+    wh->y = y1;
+    wh->lines = y2 - y1;
+    wh->cols = x2 - x1;
 
     /* dialog is empty */
     if (h->widgets == NULL)
@@ -649,10 +642,10 @@ dlg_set_position (Dlg_head * h, int y1, int x1, int y2, int x2)
         h->current = h->widgets;
 
     /* values by which controls should be moved */
-    shift_x = h->x - ox;
-    shift_y = h->y - oy;
-    scale_x = h->cols - oc;
-    scale_y = h->lines - ol;
+    shift_x = wh->x - ox;
+    shift_y = wh->y - oy;
+    scale_x = wh->cols - oc;
+    scale_y = wh->lines - ol;
 
     if ((shift_x != 0) || (shift_y != 0) || (scale_x != 0) || (scale_y != 0))
     {
@@ -669,30 +662,34 @@ dlg_set_position (Dlg_head * h, int y1, int x1, int y2, int x2)
                2. control sticks to two sides of
                one direction - it should be sized */
 
-            Widget *c = (Widget *) w->data;
+            Widget *c = WIDGET (w->data);
             int x = c->x;
             int y = c->y;
             int cols = c->cols;
             int lines = c->lines;
 
-            if ((c->pos_flags & WPOS_KEEP_LEFT) && (c->pos_flags & WPOS_KEEP_RIGHT))
+            if ((c->pos_flags & WPOS_CENTER_HORZ) != 0)
+                x = wh->x + (wh->cols - c->cols) / 2;
+            else if ((c->pos_flags & WPOS_KEEP_LEFT) != 0 && (c->pos_flags & WPOS_KEEP_RIGHT) != 0)
             {
                 x += shift_x;
                 cols += scale_x;
             }
-            else if (c->pos_flags & WPOS_KEEP_LEFT)
+            else if ((c->pos_flags & WPOS_KEEP_LEFT) != 0)
                 x += shift_x;
-            else if (c->pos_flags & WPOS_KEEP_RIGHT)
+            else if ((c->pos_flags & WPOS_KEEP_RIGHT) != 0)
                 x += shift_x + scale_x;
 
-            if ((c->pos_flags & WPOS_KEEP_TOP) && (c->pos_flags & WPOS_KEEP_BOTTOM))
+            if ((c->pos_flags & WPOS_CENTER_VERT) != 0)
+                y = wh->y + (wh->lines - c->lines) / 2;
+            else if ((c->pos_flags & WPOS_KEEP_TOP) != 0 && (c->pos_flags & WPOS_KEEP_BOTTOM) != 0)
             {
                 y += shift_y;
                 lines += scale_y;
             }
-            else if (c->pos_flags & WPOS_KEEP_TOP)
+            else if ((c->pos_flags & WPOS_KEEP_TOP) != 0)
                 y += shift_y;
-            else if (c->pos_flags & WPOS_KEEP_BOTTOM)
+            else if ((c->pos_flags & WPOS_KEEP_BOTTOM) != 0)
                 y += shift_y + scale_y;
 
             widget_set_size (c, y, x, lines, cols);
@@ -704,10 +701,10 @@ dlg_set_position (Dlg_head * h, int y1, int x1, int y2, int x2)
 /** this function sets only size, leaving positioning to automatic methods */
 
 void
-dlg_set_size (Dlg_head * h, int lines, int cols)
+dlg_set_size (WDialog * h, int lines, int cols)
 {
-    int x = h->x;
-    int y = h->y;
+    int x = WIDGET (h)->x;
+    int y = WIDGET (h)->y;
 
     if (h->flags & DLG_CENTER)
     {
@@ -725,33 +722,35 @@ dlg_set_size (Dlg_head * h, int lines, int cols)
 /** Default dialog callback */
 
 cb_ret_t
-default_dlg_callback (Dlg_head * h, Widget * sender, dlg_msg_t msg, int parm, void *data)
+dlg_default_callback (Widget * w, Widget * sender, widget_msg_t msg, int parm, void *data)
 {
+    WDialog *h = DIALOG (w);
+
     (void) sender;
     (void) parm;
     (void) data;
 
     switch (msg)
     {
-    case DLG_DRAW:
+    case MSG_DRAW:
         if (h->color != NULL)
         {
-            common_dialog_repaint (h);
+            dlg_default_repaint (h);
             return MSG_HANDLED;
         }
         return MSG_NOT_HANDLED;
 
-    case DLG_IDLE:
-        dlg_broadcast_msg_to (h, WIDGET_IDLE, FALSE, W_WANT_IDLE);
+    case MSG_IDLE:
+        dlg_broadcast_msg_to (h, MSG_IDLE, FALSE, W_WANT_IDLE);
         return MSG_HANDLED;
 
-    case DLG_RESIZE:
+    case MSG_RESIZE:
         /* this is default resizing mechanism */
         /* the main idea of this code is to resize dialog
            according to flags (if any of flags require automatic
            resizing, like DLG_CENTER, end after that reposition
            controls in dialog according to flags of widget) */
-        dlg_set_size (h, h->lines, h->cols);
+        dlg_set_size (h, w->lines, w->cols);
         return MSG_HANDLED;
 
     default:
@@ -763,34 +762,35 @@ default_dlg_callback (Dlg_head * h, Widget * sender, dlg_msg_t msg, int parm, vo
 
 /* --------------------------------------------------------------------------------------------- */
 
-Dlg_head *
-create_dlg (gboolean modal, int y1, int x1, int lines, int cols,
-            const int *colors, dlg_cb_fn callback, mouse_h mouse_handler,
+WDialog *
+dlg_create (gboolean modal, int y1, int x1, int lines, int cols,
+            const int *colors, widget_cb_fn callback, mouse_h mouse_handler,
             const char *help_ctx, const char *title, dlg_flags_t flags)
 {
-    Dlg_head *new_d;
+    WDialog *new_d;
+    Widget *w;
+
+    new_d = g_new0 (WDialog, 1);
+    w = WIDGET (new_d);
+    widget_init (w, y1, x1, lines, cols, (callback != NULL) ? callback : dlg_default_callback,
+                 mouse_handler);
+    widget_want_cursor (w, FALSE);
 
-    new_d = g_new0 (Dlg_head, 1);
     new_d->state = DLG_CONSTRUCT;
     new_d->modal = modal;
     if (colors != NULL)
         memmove (new_d->color, colors, sizeof (dlg_colors_t));
     new_d->help_ctx = help_ctx;
-    new_d->callback = (callback != NULL) ? callback : default_dlg_callback;
-    new_d->mouse = mouse_handler;
-    new_d->x = x1;
-    new_d->y = y1;
     new_d->flags = flags;
     new_d->data = NULL;
 
     dlg_set_size (new_d, lines, cols);
-    new_d->fullscreen = (new_d->x == 0 && new_d->y == 0
-                         && new_d->cols == COLS && new_d->lines == LINES);
+    new_d->fullscreen = (w->x == 0 && w->y == 0 && w->cols == COLS && w->lines == LINES);
 
     new_d->mouse_status = MOU_UNHANDLED;
 
     /* Strip existing spaces, add one space before and after the title */
-    if (title != NULL)
+    if (title != NULL && *title != '\0')
     {
         char *t;
 
@@ -827,21 +827,14 @@ dlg_set_default_colors (void)
 /* --------------------------------------------------------------------------------------------- */
 
 void
-dlg_erase (Dlg_head * h)
+dlg_erase (WDialog * h)
 {
     if ((h != NULL) && (h->state == DLG_ACTIVE))
-        tty_fill_region (h->y, h->x, h->lines, h->cols, ' ');
-}
+    {
+        Widget *wh = WIDGET (h);
 
-/* --------------------------------------------------------------------------------------------- */
-
-void
-set_idle_proc (Dlg_head * d, int enable)
-{
-    if (enable)
-        d->flags |= DLG_WANT_IDLE;
-    else
-        d->flags &= ~DLG_WANT_IDLE;
+        tty_fill_region (wh->y, wh->x, wh->lines, wh->cols, ' ');
+    }
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -850,73 +843,58 @@ set_idle_proc (Dlg_head * d, int enable)
  */
 
 unsigned long
-add_widget_autopos (Dlg_head * h, void *w, widget_pos_flags_t pos_flags, const void *before)
+add_widget_autopos (WDialog * h, void *w, widget_pos_flags_t pos_flags, const void *before)
 {
-    Widget *widget = (Widget *) w;
+    Widget *wh = WIDGET (h);
+    Widget *widget;
 
     /* Don't accept 0 widgets */
     if (w == NULL)
         abort ();
 
-    widget->x += h->x;
-    widget->y += h->y;
+    widget = WIDGET (w);
+
+    if ((pos_flags & WPOS_CENTER_HORZ) != 0)
+        widget->x = (wh->cols - widget->cols) / 2;
+    widget->x += wh->x;
+
+    if ((pos_flags & WPOS_CENTER_VERT) != 0)
+        widget->y = (wh->lines - widget->lines) / 2;
+    widget->y += wh->y;
+
     widget->owner = h;
     widget->pos_flags = pos_flags;
     widget->id = h->widget_id++;
 
-    if ((h->flags & DLG_REVERSE) != 0)
+    if (h->widgets == NULL || before == NULL)
     {
-        if (h->widgets == NULL || before == NULL)
-        {
-            h->widgets = g_list_prepend (h->widgets, widget);
-            h->current = h->widgets;
-        }
-        else
-        {
-            GList *b;
-
-            b = g_list_find (h->widgets, before);
-
-            /* don't accept widget not from dialog. This shouldn't happen */
-            if (b == NULL)
-                abort ();
-
-            h->widgets = g_list_insert_before (h->widgets, b, widget);
-            h->current = g_list_previous (b);
-        }
+        h->widgets = g_list_append (h->widgets, widget);
+        h->current = g_list_last (h->widgets);
     }
     else
     {
-        if (h->widgets == NULL || before == NULL)
-        {
-            h->widgets = g_list_append (h->widgets, widget);
-            h->current = g_list_last (h->widgets);
-        }
+        GList *b;
+
+        b = g_list_find (h->widgets, before);
+
+        /* don't accept widget not from dialog. This shouldn't happen */
+        if (b == NULL)
+            abort ();
+
+        b = g_list_next (b);
+        h->widgets = g_list_insert_before (h->widgets, b, widget);
+        if (b != NULL)
+            h->current = g_list_previous (b);
         else
-        {
-            GList *b;
-
-            b = g_list_find (h->widgets, before);
-
-            /* don't accept widget not from dialog. This shouldn't happen */
-            if (b == NULL)
-                abort ();
-
-            b = g_list_next (b);
-            h->widgets = g_list_insert_before (h->widgets, b, widget);
-            if (b != NULL)
-                h->current = g_list_previous (b);
-            else
-                h->current = g_list_last (h->widgets);
-        }
+            h->current = g_list_last (h->widgets);
     }
 
     /* widget has been added in runtime */
     if (h->state == DLG_ACTIVE)
     {
-        send_message (widget, WIDGET_INIT, 0);
-        send_message (widget, WIDGET_DRAW, 0);
-        send_message (widget, WIDGET_FOCUS, 0);
+        send_message (widget, NULL, MSG_INIT, 0, NULL);
+        send_message (widget, NULL, MSG_DRAW, 0, NULL);
+        send_message (widget, NULL, MSG_FOCUS, 0, NULL);
     }
 
     return widget->id;
@@ -926,18 +904,18 @@ add_widget_autopos (Dlg_head * h, void *w, widget_pos_flags_t pos_flags, const v
 /** wrapper to simply add lefttop positioned controls */
 
 unsigned long
-add_widget (Dlg_head * h, void *w)
+add_widget (WDialog * h, void *w)
 {
-    return add_widget_autopos (h, w, WPOS_KEEP_LEFT | WPOS_KEEP_TOP,
+    return add_widget_autopos (h, w, WPOS_KEEP_DEFAULT,
                                h->current != NULL ? h->current->data : NULL);
 }
 
 /* --------------------------------------------------------------------------------------------- */
 
 unsigned long
-add_widget_before (Dlg_head * h, void *w, void *before)
+add_widget_before (WDialog * h, void *w, void *before)
 {
-    return add_widget_autopos (h, w, WPOS_KEEP_LEFT | WPOS_KEEP_TOP, before);
+    return add_widget_autopos (h, w, WPOS_KEEP_DEFAULT, before);
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -946,24 +924,22 @@ add_widget_before (Dlg_head * h, void *w, void *before)
 void
 del_widget (void *w)
 {
-    Dlg_head *h = ((Widget *) w)->owner;
+    WDialog *h;
     GList *d;
 
     /* Don't accept NULL widget. This shouldn't happen */
     if (w == NULL)
         abort ();
 
+    h = WIDGET (w)->owner;
+
     d = g_list_find (h->widgets, w);
     if (d == h->current)
-    {
-        if ((h->flags & DLG_REVERSE) != 0)
-            h->current = dlg_widget_prev (h, d);
-        else
-            h->current = dlg_widget_next (h, d);
-    }
+        h->current = dlg_widget_next (h, d);
 
     h->widgets = g_list_remove_link (h->widgets, d);
-    send_message (d->data, WIDGET_DESTROY, 0);
+    send_message (d->data, NULL, MSG_DESTROY, 0, NULL);
+    g_free (d->data);
     g_list_free_1 (d);
 
     /* widget has been deleted in runtime */
@@ -984,18 +960,18 @@ do_refresh (void)
     if (fast_refresh)
     {
         if ((d != NULL) && (d->data != NULL))
-            dlg_redraw ((Dlg_head *) d->data);
+            dlg_redraw (DIALOG (d->data));
     }
     else
     {
         /* Search first fullscreen dialog */
         for (; d != NULL; d = g_list_next (d))
-            if ((d->data != NULL) && ((Dlg_head *) d->data)->fullscreen)
+            if (d->data != NULL && DIALOG (d->data)->fullscreen)
                 break;
         /* back to top dialog */
         for (; d != NULL; d = g_list_previous (d))
             if (d->data != NULL)
-                dlg_redraw ((Dlg_head *) d->data);
+                dlg_redraw (DIALOG (d->data));
     }
 }
 
@@ -1003,25 +979,25 @@ do_refresh (void)
 /** broadcast a message to all the widgets in a dialog */
 
 void
-dlg_broadcast_msg (Dlg_head * h, widget_msg_t msg, gboolean reverse)
+dlg_broadcast_msg (WDialog * h, widget_msg_t msg)
 {
-    dlg_broadcast_msg_to (h, msg, reverse, 0);
+    dlg_broadcast_msg_to (h, msg, FALSE, 0);
 }
 
 /* --------------------------------------------------------------------------------------------- */
 
 gboolean
-dlg_focus (Dlg_head * h)
+dlg_focus (WDialog * h)
 {
     /* cannot focus disabled widget */
     if ((h->current != NULL) && (h->state == DLG_CONSTRUCT || h->state == DLG_ACTIVE))
     {
-        Widget *current = (Widget *) h->current->data;
+        Widget *current = WIDGET (h->current->data);
 
         if (((current->options & W_DISABLED) == 0)
-            && (send_message (current, WIDGET_FOCUS, 0) == MSG_HANDLED))
+            && (send_message (current, NULL, MSG_FOCUS, 0, NULL) == MSG_HANDLED))
         {
-            h->callback (h, current, DLG_FOCUS, 0, NULL);
+            send_message (h, current, MSG_FOCUS, 0, NULL);
             return TRUE;
         }
     }
@@ -1029,47 +1005,36 @@ dlg_focus (Dlg_head * h)
     return FALSE;
 }
 
-/* --------------------------------------------------------------------------------------------- */
-/** Return true if the windows overlap */
-
-int
-dlg_overlap (Widget * a, Widget * b)
-{
-    return !((b->x >= a->x + a->cols)
-             || (a->x >= b->x + b->cols) || (b->y >= a->y + a->lines) || (a->y >= b->y + b->lines));
-}
-
-
 /* --------------------------------------------------------------------------------------------- */
 /** Find the widget with the given callback in the dialog h */
 
 Widget *
-find_widget_type (const Dlg_head * h, callback_fn callback)
+find_widget_type (const WDialog * h, widget_cb_fn callback)
 {
     GList *w;
 
     w = g_list_find_custom (h->widgets, callback, dlg_find_widget_callback);
 
-    return (w == NULL) ? NULL : (Widget *) w->data;
+    return (w == NULL) ? NULL : WIDGET (w->data);
 }
 
 /* --------------------------------------------------------------------------------------------- */
 /** Find the widget with the given id */
 
 Widget *
-dlg_find_by_id (const Dlg_head * h, unsigned long id)
+dlg_find_by_id (const WDialog * h, unsigned long id)
 {
     GList *w;
 
     w = g_list_find_custom (h->widgets, GUINT_TO_POINTER (id), dlg_find_widget_by_id);
-    return w != NULL ? (Widget *) w->data : NULL;
+    return w != NULL ? WIDGET (w->data) : NULL;
 }
 
 /* --------------------------------------------------------------------------------------------- */
 /** Find the widget with the given id in the dialog h and select it */
 
 void
-dlg_select_by_id (const Dlg_head * h, unsigned long id)
+dlg_select_by_id (const WDialog * h, unsigned long id)
 {
     Widget *w;
 
@@ -1086,8 +1051,8 @@ dlg_select_by_id (const Dlg_head * h, unsigned long id)
 void
 dlg_select_widget (void *w)
 {
-    const Widget *widget = (Widget *) w;
-    Dlg_head *h = widget->owner;
+    Widget *widget = WIDGET (w);
+    WDialog *h = widget->owner;
 
     do_select_widget (h, g_list_find (h->widgets, widget), SELECT_EXACT);
 }
@@ -1101,8 +1066,8 @@ dlg_select_widget (void *w)
 void
 dlg_set_top_widget (void *w)
 {
-    Widget *widget = (Widget *) w;
-    Dlg_head *h = widget->owner;
+    Widget *widget = WIDGET (w);
+    WDialog *h = widget->owner;
     GList *l;
 
     l = g_list_find (h->widgets, w);
@@ -1115,10 +1080,7 @@ dlg_set_top_widget (void *w)
 
     /* widget reordering */
     h->widgets = g_list_remove_link (h->widgets, l);
-    if ((h->flags & DLG_REVERSE) != 0)
-        h->widgets = g_list_concat (l, h->widgets);
-    else
-        h->widgets = g_list_concat (h->widgets, l);
+    h->widgets = g_list_concat (h->widgets, l);
     h->current = l;
 }
 
@@ -1126,7 +1088,7 @@ dlg_set_top_widget (void *w)
 /** Try to select previous widget in the tab order */
 
 void
-dlg_one_up (Dlg_head * h)
+dlg_one_up (WDialog * h)
 {
     if (h->widgets != NULL)
         do_select_widget (h, dlg_widget_prev (h, h->current), SELECT_PREV);
@@ -1136,7 +1098,7 @@ dlg_one_up (Dlg_head * h)
 /** Try to select next widget in the tab order */
 
 void
-dlg_one_down (Dlg_head * h)
+dlg_one_down (WDialog * h)
 {
     if (h->widgets != NULL)
         do_select_widget (h, dlg_widget_next (h, h->current), SELECT_NEXT);
@@ -1145,7 +1107,7 @@ dlg_one_down (Dlg_head * h)
 /* --------------------------------------------------------------------------------------------- */
 
 void
-update_cursor (Dlg_head * h)
+update_cursor (WDialog * h)
 {
     GList *p = h->current;
 
@@ -1153,10 +1115,10 @@ update_cursor (Dlg_head * h)
     {
         Widget *w;
 
-        w = (Widget *) p->data;
+        w = WIDGET (p->data);
 
         if (((w->options & W_DISABLED) == 0) && ((w->options & W_WANT_CURSOR) != 0))
-            send_message (w, WIDGET_CURSOR, 0);
+            send_message (w, NULL, MSG_CURSOR, 0, NULL);
         else
             do
             {
@@ -1164,10 +1126,10 @@ update_cursor (Dlg_head * h)
                 if (p == h->current)
                     break;
 
-                w = (Widget *) p->data;
+                w = WIDGET (p->data);
 
                 if (((w->options & W_DISABLED) == 0) && ((w->options & W_WANT_CURSOR) != 0))
-                    if (send_message (w, WIDGET_CURSOR, 0) == MSG_HANDLED)
+                    if (send_message (w, NULL, MSG_CURSOR, 0, NULL) == MSG_HANDLED)
                         break;
             }
             while (TRUE);
@@ -1181,7 +1143,7 @@ update_cursor (Dlg_head * h)
  */
 
 void
-dlg_redraw (Dlg_head * h)
+dlg_redraw (WDialog * h)
 {
     if (h->state != DLG_ACTIVE)
         return;
@@ -1189,18 +1151,18 @@ dlg_redraw (Dlg_head * h)
     if (h->winch_pending)
     {
         h->winch_pending = FALSE;
-        h->callback (h, NULL, DLG_RESIZE, 0, NULL);
+        send_message (h, NULL, MSG_RESIZE, 0, NULL);
     }
 
-    h->callback (h, NULL, DLG_DRAW, 0, NULL);
-    dlg_broadcast_msg (h, WIDGET_DRAW, (h->flags & DLG_REVERSE) != 0);
+    send_message (h, NULL, MSG_DRAW, 0, NULL);
+    dlg_broadcast_msg (h, MSG_DRAW);
     update_cursor (h);
 }
 
 /* --------------------------------------------------------------------------------------------- */
 
 void
-dlg_stop (Dlg_head * h)
+dlg_stop (WDialog * h)
 {
     h->state = DLG_CLOSED;
 }
@@ -1209,9 +1171,9 @@ dlg_stop (Dlg_head * h)
 /** Init the process */
 
 void
-init_dlg (Dlg_head * h)
+dlg_init (WDialog * h)
 {
-    if ((top_dlg != NULL) && ((Dlg_head *) top_dlg->data)->modal)
+    if (top_dlg != NULL && DIALOG (top_dlg->data)->modal)
         h->modal = TRUE;
 
     /* add dialog to the stack */
@@ -1223,8 +1185,8 @@ init_dlg (Dlg_head * h)
         if (!h->modal)
             dialog_switch_add (h);
 
-        h->callback (h, NULL, DLG_INIT, 0, NULL);
-        dlg_broadcast_msg (h, WIDGET_INIT, FALSE);
+        send_message (h, NULL, MSG_INIT, 0, NULL);
+        dlg_broadcast_msg (h, MSG_INIT);
         dlg_read_history (h);
     }
 
@@ -1242,12 +1204,12 @@ init_dlg (Dlg_head * h)
 /* --------------------------------------------------------------------------------------------- */
 
 void
-dlg_process_event (Dlg_head * h, int key, Gpm_Event * event)
+dlg_process_event (WDialog * h, int key, Gpm_Event * event)
 {
     if (key == EV_NONE)
     {
         if (tty_got_interrupt ())
-            if (h->callback (h, NULL, DLG_ACTION, CK_Cancel, NULL) != MSG_HANDLED)
+            if (send_message (h, NULL, MSG_ACTION, CK_Cancel, NULL) != MSG_HANDLED)
                 dlg_execute_cmd (h, CK_Cancel);
 
         return;
@@ -1260,20 +1222,19 @@ dlg_process_event (Dlg_head * h, int key, Gpm_Event * event)
 }
 
 /* --------------------------------------------------------------------------------------------- */
-/** Shutdown the run_dlg */
+/** Shutdown the dlg_run */
 
 void
-dlg_run_done (Dlg_head * h)
+dlg_run_done (WDialog * h)
 {
     top_dlg = g_list_remove (top_dlg, h);
 
     if (h->state == DLG_CLOSED)
     {
-        h->callback (h, (Widget *) h->current->data, DLG_END, 0, NULL);
+        send_message (h, h->current->data, MSG_END, 0, NULL);
         if (!h->modal)
             dialog_switch_remove (h);
     }
-
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -1285,10 +1246,10 @@ dlg_run_done (Dlg_head * h)
  */
 
 int
-run_dlg (Dlg_head * h)
+dlg_run (WDialog * h)
 {
-    init_dlg (h);
-    frontend_run_dlg (h);
+    dlg_init (h);
+    frontend_dlg_run (h);
     dlg_run_done (h);
     return h->ret_value;
 }
@@ -1296,11 +1257,11 @@ run_dlg (Dlg_head * h)
 /* --------------------------------------------------------------------------------------------- */
 
 void
-destroy_dlg (Dlg_head * h)
+dlg_destroy (WDialog * h)
 {
     /* if some widgets have history, save all history at one moment here */
     dlg_save_history (h);
-    dlg_broadcast_msg (h, WIDGET_DESTROY, FALSE);
+    dlg_broadcast_msg (h, MSG_DESTROY);
     g_list_foreach (h->widgets, (GFunc) g_free, NULL);
     g_list_free (h->widgets);
     mc_event_group_del (h->event_group);
@@ -1317,7 +1278,7 @@ destroy_dlg (Dlg_head * h)
   * Write history to the ${XDG_CACHE_HOME}/mc/history file
   */
 void
-dlg_save_history (Dlg_head * h)
+dlg_save_history (WDialog * h)
 {
     char *profile;
     int i;
@@ -1335,7 +1296,7 @@ dlg_save_history (Dlg_head * h)
     {
         ev_history_load_save_t event_data;
 
-        event_data.cfg = mc_config_init (profile);
+        event_data.cfg = mc_config_init (profile, FALSE);
         event_data.receiver = NULL;
 
         /* get all histories in dialog */
@@ -1351,7 +1312,7 @@ dlg_save_history (Dlg_head * h)
 /* --------------------------------------------------------------------------------------------- */
 
 char *
-dlg_get_title (const Dlg_head * h, size_t len)
+dlg_get_title (const WDialog * h, size_t len)
 {
     char *t;
 
@@ -1367,39 +1328,3 @@ dlg_get_title (const Dlg_head * h, size_t len)
 }
 
 /* --------------------------------------------------------------------------------------------- */
-/** Replace widget old_w for widget new_w in the dialog */
-
-void
-dlg_replace_widget (Widget * old_w, Widget * new_w)
-{
-    Dlg_head *h = old_w->owner;
-    gboolean should_focus = FALSE;
-
-    if (h->widgets == NULL)
-        return;
-
-    if (h->current == NULL)
-        h->current = h->widgets;
-
-    if (old_w == h->current->data)
-        should_focus = TRUE;
-
-    new_w->owner = h;
-    new_w->id = old_w->id;
-
-    if (should_focus)
-        h->current->data = new_w;
-    else
-        g_list_find (h->widgets, old_w)->data = new_w;
-
-    send_message (old_w, WIDGET_DESTROY, 0);
-    send_message (new_w, WIDGET_INIT, 0);
-
-    if (should_focus)
-        dlg_select_widget (new_w);
-
-    if (new_w->owner->state == DLG_ACTIVE)
-        send_message (new_w, WIDGET_DRAW, 0);
-}
-
-/* --------------------------------------------------------------------------------------------- */
