@@ -2,7 +2,7 @@
    Widgets for the Midnight Commander
 
    Copyright (C) 1994, 1995, 1996, 1998, 1999, 2000, 2001, 2002, 2003,
-   2004, 2005, 2006, 2007, 2009, 2010, 2011
+   2004, 2005, 2006, 2007, 2009, 2010, 2011, 2013
    The Free Software Foundation, Inc.
 
    Authors:
@@ -11,7 +11,7 @@
    Jakub Jelinek, 1995
    Andrej Borsenkow, 1996
    Norbert Warmuth, 1997
-   Andrew Borodin <aborodin@vmail.ru>, 2009, 2010
+   Andrew Borodin <aborodin@vmail.ru>, 2009, 2010, 2013
 
    This file is part of the Midnight Commander.
 
@@ -83,31 +83,32 @@ listbox_entry_free (void *data)
 static void
 listbox_drawscroll (WListbox * l)
 {
-    const int max_line = l->widget.lines - 1;
+    Widget *w = WIDGET (l);
+    int max_line = w->lines - 1;
     int line = 0;
     int i;
 
     /* Are we at the top? */
-    widget_move (&l->widget, 0, l->widget.cols);
+    widget_move (w, 0, w->cols);
     if (l->top == 0)
         tty_print_one_vline (TRUE);
     else
         tty_print_char ('^');
 
     /* Are we at the bottom? */
-    widget_move (&l->widget, max_line, l->widget.cols);
-    if ((l->top + l->widget.lines == l->count) || (l->widget.lines >= l->count))
+    widget_move (w, max_line, w->cols);
+    if ((l->top + w->lines == l->count) || (w->lines >= l->count))
         tty_print_one_vline (TRUE);
     else
         tty_print_char ('v');
 
     /* Now draw the nice relative pointer */
     if (l->count != 0)
-        line = 1 + ((l->pos * (l->widget.lines - 2)) / l->count);
+        line = 1 + ((l->pos * (w->lines - 2)) / l->count);
 
     for (i = 1; i < max_line; i++)
     {
-        widget_move (&l->widget, i, l->widget.cols);
+        widget_move (w, i, w->cols);
         if (i != line)
             tty_print_one_vline (TRUE);
         else
@@ -120,12 +121,17 @@ listbox_drawscroll (WListbox * l)
 static void
 listbox_draw (WListbox * l, gboolean focused)
 {
-    const Dlg_head *h = l->widget.owner;
-    const gboolean disabled = (((Widget *) l)->options & W_DISABLED) != 0;
+    Widget *w = WIDGET (l);
+    const WDialog *h = w->owner;
+    const gboolean disabled = (w->options & W_DISABLED) != 0;
     const int normalc = disabled ? DISABLED_COLOR : h->color[DLG_COLOR_NORMAL];
-    int selc =
-        disabled ? DISABLED_COLOR : focused ? h->
-        color[DLG_COLOR_HOT_FOCUS] : h->color[DLG_COLOR_FOCUS];
+    /* *INDENT-OFF* */
+    int selc = disabled
+        ? DISABLED_COLOR
+        : focused
+            ? h->color[DLG_COLOR_HOT_FOCUS] 
+            : h->color[DLG_COLOR_FOCUS];
+    /* *INDENT-ON* */
 
     GList *le;
     int pos;
@@ -136,7 +142,7 @@ listbox_draw (WListbox * l, gboolean focused)
     /*    pos = (le == NULL) ? 0 : g_list_position (l->list, le); */
     pos = (le == NULL) ? 0 : l->top;
 
-    for (i = 0; i < l->widget.lines; i++)
+    for (i = 0; i < w->lines; i++)
     {
         const char *text;
 
@@ -149,24 +155,25 @@ listbox_draw (WListbox * l, gboolean focused)
         else
             tty_setcolor (normalc);
 
-        widget_move (&l->widget, i, 1);
+        widget_move (l, i, 1);
 
         if ((i > 0 && pos >= l->count) || (l->list == NULL) || (le == NULL))
             text = "";
         else
         {
-            WLEntry *e = (WLEntry *) le->data;
+            WLEntry *e = LENTRY (le->data);
+
             text = e->text;
             le = g_list_next (le);
             pos++;
         }
 
-        tty_print_string (str_fit_to_term (text, l->widget.cols - 2, J_LEFT_FIT));
+        tty_print_string (str_fit_to_term (text, w->cols - 2, J_LEFT_FIT));
     }
 
     l->cursor_y = sel_line;
 
-    if (l->scrollbar && (l->count > l->widget.lines))
+    if (l->scrollbar && (l->count > w->lines))
     {
         tty_setcolor (normalc);
         listbox_drawscroll (l);
@@ -183,7 +190,7 @@ listbox_check_hotkey (WListbox * l, int key)
 
     for (i = 0, le = l->list; le != NULL; i++, le = g_list_next (le))
     {
-        WLEntry *e = (WLEntry *) le->data;
+        WLEntry *e = LENTRY (le->data);
 
         if (e->hotkey == key)
             return i;
@@ -235,6 +242,7 @@ listbox_execute_cmd (WListbox * l, unsigned long command)
 {
     cb_ret_t ret = MSG_HANDLED;
     int i;
+    Widget *w = WIDGET (l);
 
     switch (command)
     {
@@ -251,18 +259,18 @@ listbox_execute_cmd (WListbox * l, unsigned long command)
         listbox_select_last (l);
         break;
     case CK_PageUp:
-        for (i = 0; (i < l->widget.lines - 1) && (l->pos > 0); i++)
+        for (i = 0; (i < w->lines - 1) && (l->pos > 0); i++)
             listbox_back (l);
         break;
     case CK_PageDown:
-        for (i = 0; (i < l->widget.lines - 1) && (l->pos < l->count - 1); i++)
+        for (i = 0; (i < w->lines - 1) && (l->pos < l->count - 1); i++)
             listbox_fwd (l);
         break;
     case CK_Delete:
         if (l->deletable)
         {
             gboolean is_last = (l->pos + 1 >= l->count);
-            gboolean is_more = (l->top + l->widget.lines >= l->count);
+            gboolean is_more = (l->top + w->lines >= l->count);
 
             listbox_remove_current (l);
             if ((l->top > 0) && (is_last || is_more))
@@ -302,7 +310,7 @@ listbox_key (WListbox * l, int key)
         listbox_select_entry (l, key - '0');
 
         /* need scroll to item? */
-        if (abs (oldpos - l->pos) > l->widget.lines)
+        if (abs (oldpos - l->pos) > WIDGET (l)->lines)
             l->top = l->pos;
 
         return MSG_HANDLED;
@@ -358,18 +366,18 @@ listbox_destroy (WListbox * l)
 /* --------------------------------------------------------------------------------------------- */
 
 static cb_ret_t
-listbox_callback (Widget * w, widget_msg_t msg, int parm)
+listbox_callback (Widget * w, Widget * sender, widget_msg_t msg, int parm, void *data)
 {
-    WListbox *l = (WListbox *) w;
-    Dlg_head *h = l->widget.owner;
+    WListbox *l = LISTBOX (w);
+    WDialog *h = w->owner;
     cb_ret_t ret_code;
 
     switch (msg)
     {
-    case WIDGET_INIT:
+    case MSG_INIT:
         return MSG_HANDLED;
 
-    case WIDGET_HOTKEY:
+    case MSG_HOTKEY:
         {
             int pos, action;
 
@@ -378,7 +386,7 @@ listbox_callback (Widget * w, widget_msg_t msg, int parm)
                 return MSG_NOT_HANDLED;
 
             listbox_select_entry (l, pos);
-            h->callback (h, w, DLG_ACTION, l->pos, NULL);
+            send_message (h, w, MSG_ACTION, l->pos, NULL);
 
             if (l->callback != NULL)
                 action = l->callback (l);
@@ -394,38 +402,38 @@ listbox_callback (Widget * w, widget_msg_t msg, int parm)
             return MSG_HANDLED;
         }
 
-    case WIDGET_KEY:
+    case MSG_KEY:
         ret_code = listbox_key (l, parm);
         if (ret_code != MSG_NOT_HANDLED)
         {
             listbox_draw (l, TRUE);
-            h->callback (h, w, DLG_ACTION, l->pos, NULL);
+            send_message (h, w, MSG_ACTION, l->pos, NULL);
         }
         return ret_code;
 
-    case WIDGET_COMMAND:
+    case MSG_ACTION:
         return listbox_execute_cmd (l, parm);
 
-    case WIDGET_CURSOR:
-        widget_move (&l->widget, l->cursor_y, 0);
-        h->callback (h, w, DLG_ACTION, l->pos, NULL);
+    case MSG_CURSOR:
+        widget_move (l, l->cursor_y, 0);
+        send_message (h, w, MSG_ACTION, l->pos, NULL);
         return MSG_HANDLED;
 
-    case WIDGET_FOCUS:
-    case WIDGET_UNFOCUS:
-    case WIDGET_DRAW:
-        listbox_draw (l, msg != WIDGET_UNFOCUS);
+    case MSG_FOCUS:
+    case MSG_UNFOCUS:
+    case MSG_DRAW:
+        listbox_draw (l, msg != MSG_UNFOCUS);
         return MSG_HANDLED;
 
-    case WIDGET_DESTROY:
+    case MSG_DESTROY:
         listbox_destroy (l);
         return MSG_HANDLED;
 
-    case WIDGET_RESIZED:
+    case MSG_RESIZE:
         return MSG_HANDLED;
 
     default:
-        return default_proc (msg, parm);
+        return widget_default_callback (w, sender, msg, parm, data);
     }
 }
 
@@ -434,8 +442,8 @@ listbox_callback (Widget * w, widget_msg_t msg, int parm)
 static int
 listbox_event (Gpm_Event * event, void *data)
 {
-    WListbox *l = (WListbox *) data;
-    Widget *w = (Widget *) data;
+    WListbox *l = LISTBOX (data);
+    Widget *w = WIDGET (data);
 
     if (!mouse_global_in_widget (event, w))
         return MOU_UNHANDLED;
@@ -512,12 +520,14 @@ WListbox *
 listbox_new (int y, int x, int height, int width, gboolean deletable, lcback_fn callback)
 {
     WListbox *l;
+    Widget *w;
 
     if (height <= 0)
         height = 1;
 
     l = g_new (WListbox, 1);
-    init_widget (&l->widget, y, x, height, width, listbox_callback, listbox_event);
+    w = WIDGET (l);
+    widget_init (w, y, x, height, width, listbox_callback, listbox_event);
 
     l->list = NULL;
     l->top = l->pos = 0;
@@ -526,8 +536,8 @@ listbox_new (int y, int x, int height, int width, gboolean deletable, lcback_fn
     l->callback = callback;
     l->allow_duplicates = TRUE;
     l->scrollbar = !mc_global.tty.slow_terminal;
-    widget_want_hotkey (l->widget, TRUE);
-    widget_want_cursor (l->widget, FALSE);
+    widget_want_hotkey (w, TRUE);
+    widget_want_cursor (w, FALSE);
 
     return l;
 }
@@ -544,7 +554,7 @@ listbox_search_text (WListbox * l, const char *text)
 
         for (i = 0, le = l->list; le != NULL; i++, le = g_list_next (le))
         {
-            WLEntry *e = (WLEntry *) le->data;
+            WLEntry *e = LENTRY (le->data);
 
             if (strcmp (e->text, text) == 0)
                 return i;
@@ -569,8 +579,10 @@ listbox_select_first (WListbox * l)
 void
 listbox_select_last (WListbox * l)
 {
+    int lines = WIDGET (l)->lines;
+
     l->pos = l->count - 1;
-    l->top = l->count > l->widget.lines ? l->count - l->widget.lines : 0;
+    l->top = l->count > lines ? l->count - lines : 0;
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -596,8 +608,13 @@ listbox_select_entry (WListbox * l, int dest)
             l->pos = dest;
             if (!top_seen)
                 l->top = l->pos;
-            else if (l->pos - l->top >= l->widget.lines)
-                l->top = l->pos - l->widget.lines + 1;
+            else
+            {
+                int lines = WIDGET (l)->lines;
+
+                if (l->pos - l->top >= lines)
+                    l->top = l->pos - lines + 1;
+            }
             return;
         }
     }
@@ -616,7 +633,7 @@ listbox_get_current (WListbox * l, char **string, void **extra)
     gboolean ok;
 
     if (l != NULL)
-        e = (WLEntry *) g_list_nth_data (l->list, l->pos);
+        e = LENTRY (g_list_nth_data (l->list, l->pos));
 
     ok = (e != NULL);
 
@@ -638,7 +655,7 @@ listbox_remove_current (WListbox * l)
 
         current = g_list_nth (l->list, l->pos);
         l->list = g_list_remove_link (l->list, current);
-        listbox_entry_free ((WLEntry *) current->data);
+        listbox_entry_free (LENTRY (current->data));
         g_list_free_1 (current);
         l->count--;
 
