@@ -35,8 +35,10 @@
 #include <sys/types.h>
 #ifdef __FreeBSD__
 #include <sys/consio.h>
+#ifdef HAVE_SYS_IOCTL_H
 #include <sys/ioctl.h>
 #endif
+#endif
 #include <unistd.h>
 
 #include "lib/global.h"
@@ -128,6 +130,7 @@ show_console_contents_linux (int starty, unsigned char begin_line, unsigned char
 
     /* Read the value of the mc_global.tty.console_flag */
     ret = read (pipefd2[0], &message, 1);
+    (void) ret;
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -177,6 +180,7 @@ handle_console_linux (console_action_t action)
                 status = close (pipefd1[1]);
                 status = close (pipefd2[0]);
                 ret = waitpid (cons_saver_pid, &status, 0);
+                (void) ret;
             }
         }
         else
@@ -213,7 +217,7 @@ handle_console_linux (console_action_t action)
             while (0);
             mc_global.tty.console_flag = '\0';
             status = write (1, &mc_global.tty.console_flag, 1);
-            _exit (3);
+            my_exit (3);
         }                       /* if (cons_saver_pid ...) */
         break;
 
@@ -245,6 +249,7 @@ handle_console_linux (console_action_t action)
             close (pipefd2[0]);
             ret = waitpid (cons_saver_pid, &status, 0);
             mc_global.tty.console_flag = '\0';
+            (void) ret;
         }
         break;
     default:
