@@ -18,7 +18,9 @@
 
 /*** declarations of public functions ************************************************************/
 
-int display_box (WPanel * p, char **user, char **mini, int *use_msformat, int num);
+void configure_box (void);
+void panel_options_box (void);
+int panel_listing_box (WPanel * p, char **user, char **mini, int *use_msformat, int num);
 const panel_field_t *sort_box (panel_sort_info_t * info);
 void confirm_box (void);
 void display_bits_box (void);
