@@ -150,7 +150,7 @@ static gboolean is_start = FALSE;       /* Status of the start/stop toggle butto
 static char *old_dir = NULL;
 
 /* Where did we stop */
-static int resuming;
+static gboolean resuming;
 static int last_line;
 static int last_pos;
 
@@ -717,7 +717,7 @@ find_parameters (char **start_dir, ssize_t * start_dir_len,
             char *temp_dir;
 
             temp_dir = in_start->buffer;
-            if ((temp_dir[0] == '\0') || ((temp_dir[0] == '.') && (temp_dir[1] == '\0')))
+            if (*temp_dir == '\0' || DIR_IS_DOT (temp_dir))
                 temp_dir = g_strdup (vfs_path_as_str (current_panel->cwd_vpath));
             else
                 temp_dir = g_strdup (temp_dir);
@@ -768,7 +768,7 @@ find_parameters (char **start_dir, ssize_t * start_dir_len,
             s = tilde_expand (*start_dir);
             canonicalize_pathname (s);
 
-            if (s[0] == '.' && s[1] == '\0')
+            if (DIR_IS_DOT (s))
             {
                 *start_dir = g_strdup (vfs_path_as_str (current_panel->cwd_vpath));
                 /* FIXME: is current_panel->cwd_vpath canonicalized? */
@@ -792,7 +792,7 @@ find_parameters (char **start_dir, ssize_t * start_dir_len,
             }
 
             if (!options.ignore_dirs_enable || in_ignore->buffer[0] == '\0'
-                || (in_ignore->buffer[0] == '.' && in_ignore->buffer[1] == '\0'))
+                || DIR_IS_DOT (in_ignore->buffer))
                 *ignore_dirs = NULL;
             else
                 *ignore_dirs = g_strdup (in_ignore->buffer);
@@ -1068,7 +1068,7 @@ search_content (WDialog * h, const char *directory, const char *filename)
         if (resuming)
         {
             /* We've been previously suspended, start from the previous position */
-            resuming = 0;
+            resuming = FALSE;
             line = last_line;
             pos = last_pos;
         }
@@ -1106,7 +1106,7 @@ search_content (WDialog * h, const char *directory, const char *filename)
                     ret_val = TRUE;
                     break;
                 case FIND_SUSPEND:
-                    resuming = 1;
+                    resuming = TRUE;
                     last_line = line;
                     last_pos = pos;
                     ret_val = TRUE;
@@ -1306,7 +1306,7 @@ do_search (WDialog * h)
                 ;
         }                       /* while (!dp) */
 
-        if (strcmp (dp->d_name, ".") == 0 || strcmp (dp->d_name, "..") == 0)
+        if (DIR_IS_DOT (dp->d_name) || DIR_IS_DOTDOT (dp->d_name))
         {
             /* skip invalid filenames */
             while ((dp = mc_readdir (dirp)) != NULL && !str_is_valid_string (dp->d_name))
@@ -1382,7 +1382,7 @@ init_find_vars (void)
 /* --------------------------------------------------------------------------------------------- */
 
 static void
-find_do_view_edit (int unparsed_view, int edit, char *dir, char *file)
+find_do_view_edit (gboolean unparsed_view, gboolean edit, char *dir, char *file)
 {
     char *fullname = NULL;
     const char *filename = NULL;
@@ -1404,7 +1404,7 @@ find_do_view_edit (int unparsed_view, int edit, char *dir, char *file)
     if (edit)
         do_edit_at_line (fullname_vpath, use_internal_edit, line);
     else
-        view_file_at_line (fullname_vpath, unparsed_view, use_internal_view, line);
+        view_file_at_line (fullname_vpath, unparsed_view ? 1 : 0, use_internal_view, line);
     vfs_path_free (fullname_vpath);
     g_free (fullname);
 }
@@ -1412,7 +1412,7 @@ find_do_view_edit (int unparsed_view, int edit, char *dir, char *file)
 /* --------------------------------------------------------------------------------------------- */
 
 static cb_ret_t
-view_edit_currently_selected_file (int unparsed_view, int edit)
+view_edit_currently_selected_file (gboolean unparsed_view, gboolean edit)
 {
     char *dir = NULL;
     char *text = NULL;
@@ -1477,11 +1477,12 @@ find_callback (Widget * w, Widget * sender, widget_msg_t msg, int parm, void *da
     case MSG_KEY:
         if (parm == KEY_F (3) || parm == KEY_F (13))
         {
-            int unparsed_view = (parm == KEY_F (13));
-            return view_edit_currently_selected_file (unparsed_view, 0);
+            gboolean unparsed_view = (parm == KEY_F (13));
+
+            return view_edit_currently_selected_file (unparsed_view, FALSE);
         }
         if (parm == KEY_F (4))
-            return view_edit_currently_selected_file (0, 1);
+            return view_edit_currently_selected_file (FALSE, TRUE);
         return MSG_NOT_HANDLED;
 
     case MSG_RESIZE:
@@ -1530,7 +1531,7 @@ find_do_view_file (WButton * button, int action)
     (void) button;
     (void) action;
 
-    view_edit_currently_selected_file (0, 0);
+    view_edit_currently_selected_file (FALSE, FALSE);
     return 0;
 }
 
@@ -1543,7 +1544,7 @@ find_do_edit_file (WButton * button, int action)
     (void) button;
     (void) action;
 
-    view_edit_currently_selected_file (0, 1);
+    view_edit_currently_selected_file (FALSE, TRUE);
     return 0;
 }
 
@@ -1639,7 +1640,7 @@ run_process (void)
     search_file_handle->is_all_charsets = options.file_all_charsets;
     search_file_handle->is_entire_line = options.file_pattern;
 
-    resuming = 0;
+    resuming = FALSE;
 
     widget_want_idle (WIDGET (find_dlg), TRUE);
     ret = dlg_run (find_dlg);
