@@ -16,7 +16,7 @@
 
 #define MENU_PANEL (mc_global.widget.is_right ? right_panel : left_panel)
 #define MENU_PANEL_IDX  (mc_global.widget.is_right ? 1 : 0)
-#define SELECTED_IS_PANEL (get_display_type (mc_global.widget.is_right ? 1 : 0) == view_listing)
+#define SELECTED_IS_PANEL (get_display_type (MENU_PANEL_IDX) == view_listing)
 
 #define other_panel get_other_panel()
 
@@ -37,6 +37,8 @@ extern WPanel *left_panel;
 extern WPanel *right_panel;
 extern WPanel *current_panel;
 
+extern const char *mc_prompt;
+
 /*** declarations of public functions ************************************************************/
 
 void update_menu (void);
