@@ -179,6 +179,7 @@
 #endif
 
 #include "lib/global.h"
+#include "lib/strutil.h"        /* str_verscmp() */
 #include "mountlist.h"
 
 /*** global variables ****************************************************************************/
@@ -294,6 +295,7 @@ me_remote (char const *fs_name, char const *fs_type _GL_UNUSED)
 
 #ifdef STAT_STATVFS
 #if ! (__linux__ && (__GLIBC__ || __UCLIBC__))
+/* The FRSIZE fallback is not required in this case.  */
 #undef STAT_STATFS2_FRSIZE
 #else
 #include <sys/utsname.h>
@@ -313,8 +315,6 @@ me_remote (char const *fs_name, char const *fs_type _GL_UNUSED)
 #endif
 #endif /* STAT_READ_FILSYS */
 
-#include "lib/strutil.h"
-
 /*** file scope type declarations ****************************************************************/
 
 /* A mount table entry. */
@@ -359,7 +359,6 @@ static int
 statvfs_works (void)
 {
 #if ! (__linux__ && (__GLIBC__ || __UCLIBC__))
-    /* The FRSIZE fallback is not required in this case.  */
     return 1;
 #else
     static int statvfs_works_cache = -1;
@@ -368,8 +367,8 @@ statvfs_works (void)
     if (statvfs_works_cache < 0)
         statvfs_works_cache = (uname (&name) == 0 && 0 <= str_verscmp (name.release, "2.6.36"));
     return statvfs_works_cache;
-}
 #endif
+}
 #endif
 
 /* --------------------------------------------------------------------------------------------- */
@@ -779,10 +778,10 @@ read_file_system_list (int need_fs_type)
                 char *name;
                 struct stat statbuf;
 
-                if (strcmp (d->d_name, "..") == 0)
+                if (DIR_IS_DOT (d->d_name))
                     continue;
 
-                if (strcmp (d->d_name, ".") == 0)
+                if (DIR_IS_DOTDOT (d->d_name))
                     name = g_strdup ("/");
                 else
                     name = g_strconcat ("/", d->d_name, (char *) NULL);
@@ -1610,7 +1609,7 @@ my_statfs (struct my_statfs *myfs_stats, const char *path)
          ** This is the "other side" of the hack to read_file_system_list() in
          ** mountlist.c.
          ** It's not the most efficient approach, but consumes less memory. It
-         ** also accomodates QNX's ability to mount filesystems on the fly.
+         ** also accommodates QNX's ability to mount filesystems on the fly.
          */
         struct mount_entry *entry;
     struct fs_usage fs_use;
