@@ -2,9 +2,12 @@
    User Menu implementation
 
    Copyright (C) 1994, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-   2006, 2007, 2011
+   2006, 2007, 2011, 2013
    The Free Software Foundation, Inc.
 
+   Written by:
+   Slava Zanko <slavazanko@gmail.com>, 2013
+
    This file is part of the Midnight Commander.
 
    The Midnight Commander is free software: you can redistribute it
@@ -266,13 +269,8 @@ test_condition (WEdit * edit_widget, char *p, int *condition)
             break;
         case 'd':
             p = extract_arg (p, arg, sizeof (arg));
-            {
-                char *cwd_str;
-
-                cwd_str = vfs_path_to_str (panel->cwd_vpath);
-                *condition = panel != NULL && mc_search (arg, cwd_str, search_type) ? 1 : 0;
-                g_free (cwd_str);
-            }
+            *condition = panel != NULL
+                && mc_search (arg, vfs_path_as_str (panel->cwd_vpath), search_type) ? 1 : 0;
             break;
         case 't':
             p = extract_arg (p, arg, sizeof (arg));
@@ -474,7 +472,10 @@ execute_menu_command (WEdit * edit_widget, const char *commands, gboolean show_p
                 char *tmp;
                 *parameter = 0;
                 parameter =
-                    input_dialog (_("Parameter"), lc_prompt, MC_HISTORY_FM_MENU_EXEC_PARAM, "");
+                    input_dialog (_("Parameter"), lc_prompt, MC_HISTORY_FM_MENU_EXEC_PARAM, "",
+                                  INPUT_COMPLETE_FILENAMES | INPUT_COMPLETE_CD |
+                                  INPUT_COMPLETE_HOSTNAMES | INPUT_COMPLETE_VARIABLES |
+                                  INPUT_COMPLETE_USERNAMES);
                 if (!parameter || !*parameter)
                 {
                     /* User canceled */
@@ -544,22 +545,16 @@ execute_menu_command (WEdit * edit_widget, const char *commands, gboolean show_p
     mc_chmod (file_name_vpath, S_IRWXU);
     if (run_view)
     {
-        char *file_name;
-
-        file_name = vfs_path_to_str (file_name_vpath);
-        mcview_viewer (file_name, NULL, 0);
-        g_free (file_name);
+        mcview_viewer (vfs_path_as_str (file_name_vpath), NULL, 0);
         dialog_switch_process_pending ();
     }
     else
     {
         /* execute the command indirectly to allow execution even
          * on no-exec filesystems. */
-        char *file_name, *cmd;
+        char *cmd;
 
-        file_name = vfs_path_to_str (file_name_vpath);
-        cmd = g_strconcat ("/bin/sh ", file_name, (char *) NULL);
-        g_free (file_name);
+        cmd = g_strconcat ("/bin/sh ", vfs_path_as_str (file_name_vpath), (char *) NULL);
         if (!show_prompt)
         {
             if (system (cmd) == -1)
@@ -748,8 +743,9 @@ expand_format (struct WEdit *edit_widget, char c, gboolean do_quote)
     if (c == '%')
         return g_strdup ("%");
 
-    if (mc_global.mc_run_mode == MC_RUN_FULL)
+    switch (mc_global.mc_run_mode)
     {
+    case MC_RUN_FULL:
         if (g_ascii_islower ((gchar) c))
             panel = current_panel;
         else
@@ -759,12 +755,19 @@ expand_format (struct WEdit *edit_widget, char c, gboolean do_quote)
             panel = other_panel;
         }
         fname = g_strdup (panel->dir.list[panel->selected].fname);
-    }
+        break;
+
 #ifdef USE_INTERNAL_EDIT
-    else if (mc_global.mc_run_mode == MC_RUN_EDITOR)
+    case MC_RUN_EDITOR:
         fname = edit_get_file_name (edit_widget);
+        break;
 #endif
 
+    default:
+        /* other modes don't use formats */
+        return g_strdup ("");
+    }
+
     if (do_quote)
         quote_func = name_quote;
     else
@@ -787,7 +790,7 @@ expand_format (struct WEdit *edit_widget, char c, gboolean do_quote)
             char *qstr;
 
             if (panel)
-                cwd = vfs_path_to_str (panel->cwd_vpath);
+                cwd = g_strdup (vfs_path_as_str (panel->cwd_vpath));
             else
                 cwd = vfs_get_current_dir ();
 
