@@ -525,6 +525,7 @@ static const global_keymap_ini_t default_diff_keymap[] = {
     {"Edit", "f4"},
     {"EditOther", "f14"},
     {"Merge", "f5"},
+    {"MergeOther", "f15"},
     {"Search", "f7"},
     {"SearchContinue", "f17"},
     {"Options", "f9"},
@@ -578,7 +579,7 @@ create_default_keymap (void)
 {
     mc_config_t *keymap;
 
-    keymap = mc_config_init (NULL);
+    keymap = mc_config_init (NULL, TRUE);
 
     create_default_keymap_section (keymap, KEYMAP_SECTION_MAIN, default_main_keymap);
     create_default_keymap_section (keymap, KEYMAP_SECTION_MAIN_EXT, default_main_x_keymap);
