@@ -3,14 +3,14 @@
    shell connections.
 
    Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006,
-   2007, 2008, 2009, 2010, 2011
+   2007, 2008, 2009, 2010, 2011, 2013
    The Free Software Foundation, Inc.
 
    Written by:
    Pavel Machek, 1998
    Michal Svec, 2000
    Andrew Borodin <aborodin@vmail.ru>, 2010
-   Slava Zanko <slavazanko@gmail.com>, 2010
+   Slava Zanko <slavazanko@gmail.com>, 2010, 2013
    Ilia Maslakov <il.smind@gmail.com>, 2010
 
    Derived from ftpfs.c.
@@ -66,6 +66,7 @@
 #include "lib/strescape.h"
 #include "lib/unixcompat.h"
 #include "lib/fileloc.h"
+#include "lib/util.h"           /* my_exit() */
 #include "lib/mcconfig.h"
 
 #include "src/execute.h"        /* pre_exec, post_exec */
@@ -192,16 +193,17 @@ fish_load_script_from_file (const char *hostname, const char *script_name, const
 /* --------------------------------------------------------------------------------------------- */
 
 static int
-fish_decode_reply (char *s, int was_garbage)
+fish_decode_reply (char *s, gboolean was_garbage)
 {
     int code;
-    if (!sscanf (s, "%d", &code))
+
+    if (sscanf (s, "%d", &code) == 0)
     {
         code = 500;
         return 5;
     }
     if (code < 100)
-        return was_garbage ? ERROR : (!code ? COMPLETE : PRELIM);
+        return was_garbage ? ERROR : (code == 0 ? COMPLETE : PRELIM);
     return code / 100;
 }
 
@@ -212,25 +214,23 @@ static int
 fish_get_reply (struct vfs_class *me, int sock, char *string_buf, int string_len)
 {
     char answer[BUF_1K];
-    int was_garbage = 0;
+    gboolean was_garbage = FALSE;
 
-    for (;;)
+    while (TRUE)
     {
         if (!vfs_s_get_line (me, sock, answer, sizeof (answer), '\n'))
         {
-            if (string_buf)
-                *string_buf = 0;
+            if (string_buf != NULL)
+                *string_buf = '\0';
             return 4;
         }
 
-        if (strncmp (answer, "### ", 4))
-        {
-            was_garbage = 1;
-            if (string_buf)
-                g_strlcpy (string_buf, answer, string_len);
-        }
-        else
-            return fish_decode_reply (answer + 4, was_garbage);
+        if (strncmp (answer, "### ", 4) == 0)
+            return fish_decode_reply (answer + 4, was_garbage ? 1 : 0);
+
+        was_garbage = TRUE;
+        if (string_buf != NULL)
+            g_strlcpy (string_buf, answer, string_len);
     }
 }
 
@@ -254,6 +254,7 @@ fish_command (struct vfs_class *me, struct vfs_s_super *super, int wait_reply, c
         size_t ret;
         ret = fwrite (str, strlen (str), 1, logfile);
         ret = fflush (logfile);
+        (void) ret;
     }
 
     tty_enable_interrupt_key ();
@@ -340,7 +341,7 @@ fish_pipeopen (struct vfs_s_super *super, const char *path, const char *argv[])
         close (fileset2[0]);
         close (fileset2[1]);
         execvp (path, const_cast (char **, argv));
-        _exit (3);
+        my_exit (3);
     }
 }
 
@@ -520,7 +521,7 @@ fish_open_archive_int (struct vfs_class *me, struct vfs_s_super *super)
 
     vfs_print_message (_("fish: Sending initial line..."));
     /*
-     * Run `start_fish_server'. If it doesn't exist - no problem,
+     * Run 'start_fish_server'. If it doesn't exist - no problem,
      * we'll talk directly to the shell.
      */
 
@@ -779,11 +780,7 @@ fish_dir_load (struct vfs_class *me, struct vfs_s_inode *dir, char *remote_path)
                 break;
             }
         case 'S':
-#ifdef HAVE_ATOLL
-            ST.st_size = (off_t) atoll (buffer + 1);
-#else
-            ST.st_size = (off_t) atof (buffer + 1);
-#endif
+            ST.st_size = (off_t) g_ascii_strtoll (buffer + 1, NULL, 10);
             break;
         case 'P':
             {
@@ -853,10 +850,10 @@ fish_file_store (struct vfs_class *me, vfs_file_handler_t * fh, char *name, char
     fish_fh_data_t *fish = (fish_fh_data_t *) fh->data;
     gchar *shell_commands = NULL;
     struct vfs_s_super *super = FH_SUPER;
-    int n, total;
-    char buffer[8192];
+    int code;
+    off_t total;
+    char buffer[BUF_8K];
     struct stat s;
-    int was_error = 0;
     int h;
     char *quoted_name;
 
@@ -873,10 +870,10 @@ fish_file_store (struct vfs_class *me, vfs_file_handler_t * fh, char *name, char
      *
      *     ( head -c number ) | ( cat > file; cat >/dev/null )
      *
-     *  If `head' is not present on the remote system, `dd' will be used.
-     * Unfortunately, we cannot trust most non-GNU `head' implementations
-     * even if `-c' options is supported. Therefore, we separate GNU head
-     * (and other modern heads?) using `-q' and `-' . This causes another
+     *  If 'head' is not present on the remote system, 'dd' will be used.
+     * Unfortunately, we cannot trust most non-GNU 'head' implementations
+     * even if '-c' options is supported. Therefore, we separate GNU head
+     * (and other modern heads?) using '-q' and '-' . This causes another
      * implementations to fail (because of "incorrect options").
      *
      *  Fallback is:
@@ -889,7 +886,7 @@ fish_file_store (struct vfs_class *me, vfs_file_handler_t * fh, char *name, char
      *        rest=`expr $rest - $n`
      *     done
      *
-     *  `dd' was not designed for full filling of input buffers,
+     *  'dd' was not designed for full filling of input buffers,
      *  and does not report exact number of bytes (not blocks).
      *  Therefore a more complex shell script is needed.
      *
@@ -908,8 +905,8 @@ fish_file_store (struct vfs_class *me, vfs_file_handler_t * fh, char *name, char
             g_strconcat (SUP->scr_env, "FISH_FILENAME=%s FISH_FILESIZE=%" PRIuMAX ";\n",
                          SUP->scr_append, (char *) NULL);
 
-        n = fish_command (me, super, WAIT_REPLY, shell_commands, quoted_name,
-                          (uintmax_t) s.st_size);
+        code = fish_command (me, super, WAIT_REPLY, shell_commands, quoted_name,
+                             (uintmax_t) s.st_size);
         g_free (shell_commands);
     }
     else
@@ -917,11 +914,11 @@ fish_file_store (struct vfs_class *me, vfs_file_handler_t * fh, char *name, char
         shell_commands =
             g_strconcat (SUP->scr_env, "FISH_FILENAME=%s FISH_FILESIZE=%" PRIuMAX ";\n",
                          SUP->scr_send, (char *) NULL);
-        n = fish_command (me, super, WAIT_REPLY, shell_commands, quoted_name,
-                          (uintmax_t) s.st_size);
+        code = fish_command (me, super, WAIT_REPLY, shell_commands, quoted_name,
+                             (uintmax_t) s.st_size);
         g_free (shell_commands);
     }
-    if (n != PRELIM)
+    if (code != PRELIM)
     {
         close (h);
         ERRNOR (E_REMOTE, -1);
@@ -931,7 +928,8 @@ fish_file_store (struct vfs_class *me, vfs_file_handler_t * fh, char *name, char
 
     while (TRUE)
     {
-        int t;
+        ssize_t n, t;
+
         while ((n = read (h, buffer, sizeof (buffer))) < 0)
         {
             if ((errno == EINTR) && tty_got_interrupt ())
@@ -955,14 +953,13 @@ fish_file_store (struct vfs_class *me, vfs_file_handler_t * fh, char *name, char
         }
         tty_disable_interrupt_key ();
         total += n;
-        vfs_print_message ("%s: %d/%" PRIuMAX,
-                           was_error ? _("fish: storing zeros") : _("fish: storing file"),
-                           total, (uintmax_t) s.st_size);
+        vfs_print_message ("%s: %" PRIuMAX "/%" PRIuMAX, _("fish: storing file"),
+                           (uintmax_t) total, (uintmax_t) s.st_size);
     }
     close (h);
     g_free (quoted_name);
 
-    if ((fish_get_reply (me, SUP->sockr, NULL, 0) != COMPLETE) || was_error)
+    if (fish_get_reply (me, SUP->sockr, NULL, 0) != COMPLETE)
         ERRNOR (E_REMOTE, -1);
     return 0;
 
@@ -997,9 +994,9 @@ fish_linear_start (struct vfs_class *me, vfs_file_handler_t * fh, off_t offset)
     fish->append = FALSE;
 
     /*
-     * Check whether the remote file is readable by using `dd' to copy 
-     * a single byte from the remote file to /dev/null. If `dd' completes
-     * with exit status of 0 use `cat' to send the file contents to the
+     * Check whether the remote file is readable by using 'dd' to copy 
+     * a single byte from the remote file to /dev/null. If 'dd' completes
+     * with exit status of 0 use 'cat' to send the file contents to the
      * standard output (i.e. over the network).
      */
 
@@ -1017,7 +1014,7 @@ fish_linear_start (struct vfs_class *me, vfs_file_handler_t * fh, off_t offset)
 #if SIZEOF_OFF_T == SIZEOF_LONG
     fish->total = (off_t) strtol (reply_str, NULL, 10);
 #else
-    fish->total = (off_t) strtoll (reply_str, NULL, 10);
+    fish->total = (off_t) g_ascii_strtoll (reply_str, NULL, 10);
 #endif
     if (errno != 0)
         ERRNOR (E_REMOTE, 0);
@@ -1032,13 +1029,13 @@ fish_linear_abort (struct vfs_class *me, vfs_file_handler_t * fh)
     fish_fh_data_t *fish = (fish_fh_data_t *) fh->data;
     struct vfs_s_super *super = FH_SUPER;
     char buffer[BUF_8K];
-    int n;
+    ssize_t n;
 
     vfs_print_message (_("Aborting transfer..."));
 
     do
     {
-        n = MIN (sizeof (buffer), (size_t) (fish->total - fish->got));
+        n = MIN ((off_t) sizeof (buffer), (fish->total - fish->got));
         if (n != 0)
         {
             n = read (SUP->sockr, buffer, n);
@@ -1057,14 +1054,13 @@ fish_linear_abort (struct vfs_class *me, vfs_file_handler_t * fh)
 
 /* --------------------------------------------------------------------------------------------- */
 
-static int
+static ssize_t
 fish_linear_read (struct vfs_class *me, vfs_file_handler_t * fh, void *buf, size_t len)
 {
     fish_fh_data_t *fish = (fish_fh_data_t *) fh->data;
     struct vfs_s_super *super = FH_SUPER;
     ssize_t n = 0;
 
-
     len = MIN ((size_t) (fish->total - fish->got), len);
     tty_disable_interrupt_key ();
     while (len != 0 && ((n = read (SUP->sockr, buf, len)) < 0))
@@ -1525,7 +1521,7 @@ fish_fh_open (struct vfs_class *me, vfs_file_handler_t * fh, int flags, mode_t m
                 vfs_path_free (vpath);
                 goto fail;
             }
-            fh->ino->localname = vfs_path_to_str (vpath);
+            fh->ino->localname = g_strdup (vfs_path_as_str (vpath));
             vfs_path_free (vpath);
             close (tmp_handle);
         }
