@@ -67,8 +67,9 @@ local_open (const vfs_path_t * vpath, int flags, mode_t mode)
 {
     int *local_info;
     int fd;
-    vfs_path_element_t *path_element = vfs_path_get_by_index (vpath, -1);
+    const vfs_path_element_t *path_element;
 
+    path_element = vfs_path_get_by_index (vpath, -1);
     fd = open (path_element->path, NO_LINEAR (flags), mode);
     if (fd == -1)
         return 0;
@@ -86,8 +87,9 @@ local_opendir (const vfs_path_t * vpath)
 {
     DIR **local_info;
     DIR *dir;
-    vfs_path_element_t *path_element = vfs_path_get_by_index (vpath, -1);
+    const vfs_path_element_t *path_element;
 
+    path_element = vfs_path_get_by_index (vpath, -1);
     dir = opendir (path_element->path);
     if (!dir)
         return 0;
@@ -123,8 +125,9 @@ local_closedir (void *data)
 static int
 local_stat (const vfs_path_t * vpath, struct stat *buf)
 {
-    vfs_path_element_t *path_element = vfs_path_get_by_index (vpath, -1);
+    const vfs_path_element_t *path_element;
 
+    path_element = vfs_path_get_by_index (vpath, -1);
     return stat (path_element->path, buf);
 }
 
@@ -133,8 +136,9 @@ local_stat (const vfs_path_t * vpath, struct stat *buf)
 static int
 local_lstat (const vfs_path_t * vpath, struct stat *buf)
 {
-    vfs_path_element_t *path_element = vfs_path_get_by_index (vpath, -1);
+    const vfs_path_element_t *path_element;
 
+    path_element = vfs_path_get_by_index (vpath, -1);
 #ifndef HAVE_STATLSTAT
     return lstat (path_element->path, buf);
 #else
@@ -147,8 +151,9 @@ local_lstat (const vfs_path_t * vpath, struct stat *buf)
 static int
 local_chmod (const vfs_path_t * vpath, mode_t mode)
 {
-    vfs_path_element_t *path_element = vfs_path_get_by_index (vpath, -1);
+    const vfs_path_element_t *path_element;
 
+    path_element = vfs_path_get_by_index (vpath, -1);
     return chmod (path_element->path, mode);
 }
 
@@ -157,8 +162,9 @@ local_chmod (const vfs_path_t * vpath, mode_t mode)
 static int
 local_chown (const vfs_path_t * vpath, uid_t owner, gid_t group)
 {
-    vfs_path_element_t *path_element = vfs_path_get_by_index (vpath, -1);
+    const vfs_path_element_t *path_element;
 
+    path_element = vfs_path_get_by_index (vpath, -1);
     return chown (path_element->path, owner, group);
 }
 
@@ -167,8 +173,9 @@ local_chown (const vfs_path_t * vpath, uid_t owner, gid_t group)
 static int
 local_utime (const vfs_path_t * vpath, struct utimbuf *times)
 {
-    vfs_path_element_t *path_element = vfs_path_get_by_index (vpath, -1);
+    const vfs_path_element_t *path_element;
 
+    path_element = vfs_path_get_by_index (vpath, -1);
     return utime (path_element->path, times);
 }
 
@@ -177,8 +184,9 @@ local_utime (const vfs_path_t * vpath, struct utimbuf *times)
 static int
 local_readlink (const vfs_path_t * vpath, char *buf, size_t size)
 {
-    vfs_path_element_t *path_element = vfs_path_get_by_index (vpath, -1);
+    const vfs_path_element_t *path_element;
 
+    path_element = vfs_path_get_by_index (vpath, -1);
     return readlink (path_element->path, buf, size);
 }
 
@@ -187,8 +195,9 @@ local_readlink (const vfs_path_t * vpath, char *buf, size_t size)
 static int
 local_unlink (const vfs_path_t * vpath)
 {
-    vfs_path_element_t *path_element = vfs_path_get_by_index (vpath, -1);
+    const vfs_path_element_t *path_element;
 
+    path_element = vfs_path_get_by_index (vpath, -1);
     return unlink (path_element->path);
 }
 
@@ -197,9 +206,11 @@ local_unlink (const vfs_path_t * vpath)
 static int
 local_symlink (const vfs_path_t * vpath1, const vfs_path_t * vpath2)
 {
-    vfs_path_element_t *path_element1 = vfs_path_get_by_index (vpath1, -1);
-    vfs_path_element_t *path_element2 = vfs_path_get_by_index (vpath2, -1);
+    const vfs_path_element_t *path_element1;
+    const vfs_path_element_t *path_element2;
 
+    path_element1 = vfs_path_get_by_index (vpath1, -1);
+    path_element2 = vfs_path_get_by_index (vpath2, -1);
     return symlink (path_element1->path, path_element2->path);
 }
 
@@ -235,9 +246,11 @@ local_write (void *data, const char *buf, size_t nbyte)
 static int
 local_rename (const vfs_path_t * vpath1, const vfs_path_t * vpath2)
 {
-    vfs_path_element_t *path_element1 = vfs_path_get_by_index (vpath1, -1);
-    vfs_path_element_t *path_element2 = vfs_path_get_by_index (vpath2, -1);
+    const vfs_path_element_t *path_element1;
+    const vfs_path_element_t *path_element2;
 
+    path_element1 = vfs_path_get_by_index (vpath1, -1);
+    path_element2 = vfs_path_get_by_index (vpath2, -1);
     return rename (path_element1->path, path_element2->path);
 }
 
@@ -246,8 +259,9 @@ local_rename (const vfs_path_t * vpath1, const vfs_path_t * vpath2)
 static int
 local_chdir (const vfs_path_t * vpath)
 {
-    vfs_path_element_t *path_element = vfs_path_get_by_index (vpath, -1);
+    const vfs_path_element_t *path_element;
 
+    path_element = vfs_path_get_by_index (vpath, -1);
     return chdir (path_element->path);
 }
 
@@ -256,8 +270,9 @@ local_chdir (const vfs_path_t * vpath)
 static int
 local_mknod (const vfs_path_t * vpath, mode_t mode, dev_t dev)
 {
-    vfs_path_element_t *path_element = vfs_path_get_by_index (vpath, -1);
+    const vfs_path_element_t *path_element;
 
+    path_element = vfs_path_get_by_index (vpath, -1);
     return mknod (path_element->path, mode, dev);
 }
 
@@ -266,9 +281,11 @@ local_mknod (const vfs_path_t * vpath, mode_t mode, dev_t dev)
 static int
 local_link (const vfs_path_t * vpath1, const vfs_path_t * vpath2)
 {
-    vfs_path_element_t *path_element1 = vfs_path_get_by_index (vpath1, -1);
-    vfs_path_element_t *path_element2 = vfs_path_get_by_index (vpath2, -1);
+    const vfs_path_element_t *path_element1;
+    const vfs_path_element_t *path_element2;
 
+    path_element1 = vfs_path_get_by_index (vpath1, -1);
+    path_element2 = vfs_path_get_by_index (vpath2, -1);
     return link (path_element1->path, path_element2->path);
 }
 
@@ -277,8 +294,9 @@ local_link (const vfs_path_t * vpath1, const vfs_path_t * vpath2)
 static int
 local_mkdir (const vfs_path_t * vpath, mode_t mode)
 {
-    vfs_path_element_t *path_element = vfs_path_get_by_index (vpath, -1);
+    const vfs_path_element_t *path_element;
 
+    path_element = vfs_path_get_by_index (vpath, -1);
     return mkdir (path_element->path, mode);
 }
 
@@ -287,25 +305,24 @@ local_mkdir (const vfs_path_t * vpath, mode_t mode)
 static int
 local_rmdir (const vfs_path_t * vpath)
 {
-    vfs_path_element_t *path_element = vfs_path_get_by_index (vpath, -1);
+    const vfs_path_element_t *path_element;
 
+    path_element = vfs_path_get_by_index (vpath, -1);
     return rmdir (path_element->path);
 }
 
 /* --------------------------------------------------------------------------------------------- */
 
-static char *
+static vfs_path_t *
 local_getlocalcopy (const vfs_path_t * vpath)
 {
-    vfs_path_element_t *path_element = vfs_path_get_by_index (vpath, -1);
-
-    return g_strdup (path_element->path);
+    return vfs_path_clone (vpath);
 }
 
 /* --------------------------------------------------------------------------------------------- */
 
 static int
-local_ungetlocalcopy (const vfs_path_t * vpath, const char *local, int has_changed)
+local_ungetlocalcopy (const vfs_path_t * vpath, const vfs_path_t * local, gboolean has_changed)
 {
     (void) vpath;
     (void) local;
