@@ -1,11 +1,11 @@
 /*
    lib/vfs - x_basename() function testing
 
-   Copyright (C) 2011
+   Copyright (C) 2011, 2013
    The Free Software Foundation, Inc.
 
    Written by:
-   Slava Zanko <slavazanko@gmail.com>, 2011
+   Slava Zanko <slavazanko@gmail.com>, 2011, 2013
 
    This file is part of the Midnight Commander.
 
@@ -21,58 +21,91 @@
 
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
-*/
+ */
 
 #define TEST_SUITE_NAME "/lib"
 
-#include <config.h>
-
-#include <check.h>
+#include "tests/mctest.h"
 
 #include <stdio.h>
 
-#include "lib/global.h"
 #include "lib/strutil.h"
 #include "lib/util.h"
 
+/* --------------------------------------------------------------------------------------------- */
 
+/* @Before */
 static void
 setup (void)
 {
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
+/* @After */
 static void
 teardown (void)
 {
 }
 
 /* --------------------------------------------------------------------------------------------- */
-#define check_x_basename( input, etalon ) \
-{ \
-    result = x_basename ( input ); \
-    fail_unless(strcmp(result, etalon) == 0, \
-    "\ninput (%s)\nactial  (%s) not equal to\netalon (%s)", input, result, etalon); \
-}
 
-START_TEST (test_x_basename)
+/* @DataSource("test_x_basename_ds") */
+/* *INDENT-OFF* */
+static const struct test_x_basename_ds
 {
-    const char *result;
-    check_x_basename ("/test/path/test2/path2", "path2");
+    const char *input_value;
+    const char *expected_result;
+} test_x_basename_ds[] =
+{
+    {
+        "/test/path/test2/path2",
+        "path2"
+    },
+    {
+        "/test/path/test2/path2#vfsprefix",
+        "path2#vfsprefix"
+    },
+    {
+        "/test/path/test2/path2/vfsprefix://",
+        "path2/vfsprefix://"
+    },
+    {
+        "/test/path/test2/path2/vfsprefix://subdir",
+        "subdir"
+    },
+    {
+        "/test/path/test2/path2/vfsprefix://subdir/",
+        "subdir/"
+    },
+    {
+        "/test/path/test2/path2/vfsprefix://subdir/subdir2",
+        "subdir2"
+    },
+    {
+        "/test/path/test2/path2/vfsprefix:///",
+        "/"
+    },
+};
+/* *INDENT-ON* */
 
-    check_x_basename ("/test/path/test2/path2#vfsprefix", "path2#vfsprefix");
+/* @Test(dataSource = "test_x_basename_ds") */
+/* *INDENT-OFF* */
+START_PARAMETRIZED_TEST (test_x_basename, test_x_basename_ds)
+/* *INDENT-ON* */
+{
+    /* given */
+    const char *actual_result;
 
-    check_x_basename ("/test/path/test2/path2/vfsprefix://", "path2/vfsprefix://");
+    /* when */
+    actual_result = x_basename (data->input_value);
 
-
-    check_x_basename ("/test/path/test2/path2/vfsprefix://subdir", "subdir");
-
-    check_x_basename ("/test/path/test2/path2/vfsprefix://subdir/", "subdir/");
-
-    check_x_basename ("/test/path/test2/path2/vfsprefix://subdir/subdir2", "subdir2");
-
-    check_x_basename ("/test/path/test2/path2/vfsprefix:///", "/");
+    /* then */
+    mctest_assert_str_eq (actual_result, data->expected_result);
 }
-END_TEST
+/* *INDENT-OFF* */
+END_PARAMETRIZED_TEST
+/* *INDENT-ON* */
 
 /* --------------------------------------------------------------------------------------------- */
 
@@ -88,7 +121,7 @@ main (void)
     tcase_add_checked_fixture (tc_core, setup, teardown);
 
     /* Add new tests here: *************** */
-    tcase_add_test (tc_core, test_x_basename);
+    mctest_add_parameterized_test (tc_core, test_x_basename, test_x_basename_ds);
     /* *********************************** */
 
     suite_add_tcase (s, tc_core);
